// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMM_BAND
#define PPSSUMM_BAND

#include <string>
#include <cstdlib>
#include "BaseInteger.h"
#include "Dal.h"

class Band : public BaseInteger<int32>
{
    public:
    Band() : BaseInteger<int32>(0) { }
    Band( const int32& s ) : BaseInteger<int32>(s) {  }
    Band( const char* s ) : BaseInteger<int32>( static_cast<int32>( strtol( s , nullptr , 10 ) ) ) {  }
    Band( const std::string& s ) :  BaseInteger<int32>( static_cast<int32>( strtol( s.c_str() , nullptr , 10 ) ) ) {  }

    Band( const Band& s ) : BaseInteger<int32>(s.get()) { }
    Band operator=(const char* s)
    {
        this->set( static_cast<int32>( strtol( s , nullptr , 10 ) ) );
        return *this;
    }
    Band operator=(const std::string& s)
    { 
        this->set( static_cast<int32>( strtol( s.c_str() , nullptr , 10 ) ) );
        return *this;
    }
    Band operator=(const Band& s)
    { 
        this->set( s.get() );
        return *this;
    }

    std::string energy_limits() const
    {
        int32 n(this->get());
        std::string s("&nbsp;");
        if(n > 0 && n < 10)
	    s = std::string(_energy_limits[n])
	;
        return s;
    }

    private:
    static std::string _energy_limits[];
};

class MakeBand
{
    public:
    Band operator()(const Band& s) { return Band(s); }
    Band operator()(const int32& s) { return Band(s); }
    Band operator()(const char* s) { return Band(s); }
    Band operator()(const std::string& s) { return Band(s); }
};
#endif
