// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMM_BASEINTEGER
#define PPSSUMM_BASEINTEGER

#include <string>
#include <cstdlib>
#include <iostream>

template <typename T>
class BaseInteger 
{
    public:
    BaseInteger() : _base_integer(0) { }
    BaseInteger(const T& s ) : _base_integer(s) { }

    // Accessor
    T get() const { return _base_integer; }
    T set( const T& s ) 
    { 
        T tmp = _base_integer;
        _base_integer = s;
        return tmp; 
    }

    // Operators
    bool operator<(const BaseInteger& s) const
    { 
        return _base_integer < s._base_integer;
    }
    bool operator>(const BaseInteger& s) const
    { 
        return _base_integer > s._base_integer;
    }
    bool operator==(const BaseInteger& s) const
    { 
        return _base_integer == s._base_integer;
    }

    // prefix and postfix 
    BaseInteger operator=(const T& s)
    { 
        _base_integer = s;
        return *this;
    }

    //
    BaseInteger operator++()
    { 
        ++_base_integer;
        return *this;
    }
    BaseInteger operator++(int i)
    { 
        ++_base_integer;
        return *this;
    }

    BaseInteger operator--()
    { 
        --_base_integer;
        return *this;
    }
    BaseInteger operator--(int i)
    { 
        --_base_integer;
        return *this;
    }

    BaseInteger operator+=(const T& i)
    { 
        _base_integer+=i;
        return *this;
    }
    BaseInteger operator-=(const T& i)
    { 
        _base_integer-=i;
        return *this;
    }

    friend std::ostream &operator<<(std::ostream& os , const BaseInteger& s)
    {
        T tmp = s.get();
        if (!tmp)
        {
        os << "&nbsp;";
        }
        else
        {
        os << tmp;
        }
       return os;
    }
    private:
    T _base_integer;
};
#endif
