// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMM_EXTENDFITSCOLUMN
#define PPSSUMM_EXTENDFITSCOLUMN

/* 
 *
 * Extending the functions provided by FitsColumn.h
 * with an alternative interface.
 *
 */

#include <iterator>
#include "ppssumm_algo.h"
#include "FitsColumn.h"
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"

template< typename Oi_  >
Oi_ bool8Column( const Column* col, Oi_ result_ , unsigned int from = 0, unsigned int nr = 0  ) 
{

    unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
    vector<unsigned int> v;
    Tseq( from  , rows , back_inserter(v));
    return bool8Column(col , result_ , v.begin() , v.end() );

}

template< typename Oi_ >
Oi_ bool8Column( const Table* t  , const std::string& colname ,  Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{
    const Column* col = t->column(colname);
    return bool8Column( col , result_ , from , nr );
}

template< typename Oi_  >
Oi_ int8Column( const Column* col, Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0 ) 
{

    unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
    vector<unsigned int> v;
    Tseq( from  , rows , back_inserter(v));
    return int8Column(col , result_ , v.begin() , v.end() );

}

template< typename Oi_ >
Oi_ int8Column( const Table* t  , const std::string& colname ,  Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{
    const Column* col = t->column(colname);
    return int8Column( col , result_ , from , nr );
}

template< typename Oi_  >
Oi_ uint16Column( const Column* col, Oi_ result_ , unsigned int from = 0, unsigned int nr = 0  ) 
{

    unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
    vector<unsigned int> v;
    Tseq( from  , rows , back_inserter(v));
    return uint16Column(col , result_ , v.begin() , v.end() );

}

template< typename Oi_ >
Oi_ uint16Column( const Table* t  , const std::string& colname ,  Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{
    const Column* col = t->column(colname);
    return uint16Column( col , result_ , from , nr );
}

template< typename Oi_  >
Oi_ int16Column( const Column* col, Oi_ result_ , unsigned int from = 0, unsigned int nr = 0 ) 
{

    unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
    vector<unsigned int> v;
    Tseq( from  , rows , back_inserter(v));
    return int16Column(col , result_ , v.begin() , v.end() );

}

template< typename Oi_ >
Oi_ int16Column( const Table* t  , const std::string& colname ,  Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{
    const Column* col = t->column(colname);
    return int16Column( col , result_ , from , nr );
}

template< typename Oi_  >
Oi_ uint32Column( const Column* col, Oi_ result_ , unsigned int from = 0, unsigned int nr = 0 ) 
{

    unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
    vector<unsigned int> v;
    Tseq( from  , rows , back_inserter(v));
    return uint32Column(col , result_ , v.begin() , v.end() );

}

template< typename Oi_ >
Oi_ uint32Column( const Table* t  , const std::string& colname ,  Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{
    const Column* col = t->column(colname);
    return uint32Column( col , result_ , from , nr );
}

template< typename Oi_  >
Oi_ int32Column( const Column* col, Oi_ result_ , unsigned int from = 0, unsigned int nr = 0 ) 
{

    unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
    vector<unsigned int> v;
    Tseq( from  , rows , back_inserter(v));
    return int32Column(col , result_ , v.begin() , v.end() );

}

template< typename Oi_ >
Oi_ int32Column( const Table* t  , const std::string& colname ,  Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{
    const Column* col = t->column(colname);
    return int32Column( col , result_ , from , nr );
}

template< typename Oi_  >
Oi_ real32Column( const Column* col, Oi_ result_ , unsigned int from = 0, unsigned int nr = 0 ) 
{

    unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
    vector<unsigned int> v;
    Tseq( from  , rows , back_inserter(v));
    return real32Column(col , result_ , v.begin() , v.end() );

}

template< typename Oi_ >
Oi_ real32Column( const Table* t  , const std::string& colname ,  Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{
    const Column* col = t->column(colname);
    return real32Column( col , result_ , from , nr );
}

template< typename Oi_  >
Oi_ real64Column( const Column* col, Oi_ result_ , unsigned int from = 0, unsigned int nr = 0 ) 
{

    unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
    vector<unsigned int> v;
    Tseq( from  , rows , back_inserter(v));
    return real64Column(col , result_ , v.begin() , v.end() );

}

template< typename Oi_ >
Oi_ real64Column( const Table* t  , const std::string& colname ,  Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{
    const Column* col = t->column(colname);
    return real64Column( col , result_ , from , nr );
}

template< typename Oi_ >
Oi_ stringColumn( const Column* col , Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{

    unsigned int rows = (nr && ((from + nr)  <= col->parent()->rows()) ) ? nr : col->parent()->rows() - from ;
    vector<unsigned int> v;
    Tseq( from  , rows , back_inserter(v));
    return stringColumn(col , result_ , v.begin() , v.end() );

}

template< typename Oi_ >
Oi_ stringColumn( const Table* t  , const std::string& colname ,  Oi_ result_ , unsigned int from = 0 , unsigned int nr = 0  ) 
{
    const Column* col = t->column(colname);
    return stringColumn( col , result_ , from , nr );
}

#endif
