// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMM_FILTGRSP
#define PPSSUMM_FILTGRSP

#include <vector>
#include <string>
#include <sstream>
#include <cstdlib>
#include "ppssumm_typedef.h"
#include "BaseInteger.h"
#include "Dal.h"

class Filtgrps : public BaseInteger<int32>
{
    public:
    Filtgrps() : BaseInteger<int32>(0) { }
    Filtgrps( const int32& s ) : BaseInteger<int32>(s) {  }
    Filtgrps( const char* s ) : BaseInteger<int32>( static_cast<int32>( strtol( s , nullptr , 10 ) ) ) {  }
    Filtgrps( const std::string& s ) :  BaseInteger<int32>( static_cast<int32>( strtol( s.c_str() , nullptr , 10 ) ) ) {  }

    Filtgrps( const Filtgrps& s ) : BaseInteger<int32>(s.get()) { }
    Filtgrps operator=(const char* s)
    {
        this->set( static_cast<int32>( strtol( s , nullptr , 10 ) ) );
        return *this;
    }
    Filtgrps operator=(const std::string& s)
    { 
        this->set( static_cast<int32>( strtol( s.c_str() , nullptr , 10 ) ) );
        return *this;
    }
    Filtgrps operator=(const Filtgrps& s)
    { 
        this->set( s.get() );
        return *this;
    }

    friend std::ostream& operator<<( std::ostream& os , const Filtgrps& s )
    {
	stringlist text;
	long t = s.get();
	if ( t & 1 ) text.push_back("rejflags");
	if ( t & 2 ) text.push_back("attGTI");
	if ( t & 4 ) text.push_back("hkGTI");

	if (text.size())
	{
	    std::stringstream ss;
	    stringlist_ci CI= text.begin();
	    ss << *CI;
	    ++CI;
	    for ( ; CI != text.end(); ++CI )
	    {
		ss << "," << *CI;
	    }
	    os << ss.str();
	}   
	else
	{
	    os << "None";
	}
	return os;
    }
};

class MakeFiltgrps
{
    public:
    Filtgrps operator()(const Filtgrps& s) { return Filtgrps(s); }
    Filtgrps operator()(const int32& s) { return Filtgrps(s); }
    Filtgrps operator()(const char* s) { return Filtgrps(s); }
    Filtgrps operator()(const std::string& s) { return Filtgrps(s); }
};
#endif
