// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "HtmlPage.h"

HtmlPage::HtmlPage(const PpssummCommon* pc ) : _page_title()
{
    _pc = pc;
    std::string tmp_header[] = 
    {
	"<html>","\n"
	,"<head>","\n"
	,"<title>","\n"
	,":PAGE_TITLE:"
	,"</title>","\n"
	,"<style>","\n"
	,"<!-- \n"
	,"h1, h2 ,h3 { color: #000000; text-align: left; }\n"

	,"body { bgcolor: #FFFFFF; color: #000000; font-family: monospace; }\n"
	,"img { border: 0; }\n"
	//,"hr { noshade: noshade; }\n"

	,"#subtitle table { width: 90%;  border: 1em;}\n"
	,"#subtitle th { text-align: left; font-family: monospace; font-weight: bold; }\n"
	,"#subtitle td { text-align: left; font-family: monospace; font-weight: medium; }\n"

	//,"#normaltable table { width: 90%; }\n"
	,"#normaltable th.string { padding-left: 1em; text-align: left; font-weight: bold; font-family: monospace }\n"
	,"#normaltable th.date { padding-left: 1em; text-align: left; font-family: monospace; font-weight: bold; }\n"
	,"#normaltable th.unit { padding-left: 1em; text-align: left; font-family: monospace; font-size: small; font-weight: bold; }\n"
	,"#normaltable th.angle { padding-left: 1em; text-align: left; font-family: monospace; font-weight: bold; }\n"
	,"#normaltable th.number { padding-left: 1em; text-align: left; font-family: monospace; font-weight: bold; }\n"
	,"#normaltable td.string { padding-left: 1em; text-align: left; font-family: monospace; font-weight: medium; }\n"
	,"#normaltable td.date { padding-left: 1em; text-align: left; font-family: monospace; font-weight: medium; }\n"
	,"#normaltable td.number { padding-left: 1em; text-align: right; font-family: monospace; font-weight: medium;}\n"
	,"#normaltable td.angle { padding-left: 1em; text-align: right; font-family: monospace; font-weight: medium; }\n"

	//,"#widetable table { width: 90%; }\n"
	,"#widetable th.string { padding-left: 1em; text-align: left; font-family: monospace; font-weight: bold; }\n"
	,"#widetable th.flags { padding-left: 1em; text-align: left; font-family: monospace; font-weight: bold; }\n"
	,"#widetable th.date { padding-left: 1em; text-align: left; font-family: monospace; font-weight: bold; }\n"
	,"#widetable th.flag { padding-left: 1em; text-align: center; font-family: monospace; font-weight: bold; }\n"
	,"#widetable th.angle { padding-left: 1em; text-align: left; font-family: monospace; font-weight: bold; }\n"
	,"#widetable th.number { padding-left: 1em; text-align: left; font-family: monospace; font-weight: bold; }\n"
	,"#widetable th.unit { padding-left: 1em; text-align: left; font-family: monospace; font-size: small; font-weight: bold; }\n"
	,"#widetable td.string { padding-left: 1em; padding-left: 1em; text-align: left; font-family: monospace; font-size: small; font-weight: medium; }\n"
	,"#widetable td.flags { padding-left: 1em; text-align: left; font-family: monospace; font-size: small; font-weight: medium; font-stretch: narrower; }\n"
	,"#widetable td.flag { padding-left: 1em; text-align: center; font-family: monospace; font-size: small; font-weight: medium; font-stretch: narrower; }\n"
	,"#widetable td.date { padding-left: 1em; text-align: left; font-family: monospace; font-size: small; font-weight: medium; }\n"
	,"#widetable td.number { padding-left: 1em; text-align: right; font-family: monospace; font-size: small; font-weight: medium; }\n"
	,"#widetable td.angle { padding-left: 1em; text-align: right; font-family: monospace; font-size: small; font-weight: medium; }\n"

	,"a { text-decoration:none; color:#000000; }\n" 
	,"a:link { text-decoration:none; color:#0000FF; }\n"
	,"a:active { text-decoration:none; color:#00FF00; }\n"
	,"a:visited { text-decoration:none; color:#000077; }\n"
	,"a:hover { text-decoration:underline; color:#777777; }\n"

	,"#linkbar ul { float:left; width:100%; padding:0; margin:0; list-style-type:none; font-size: large; } \n"
	,"#linkbar a { float:left; width:8em; font-weight: bold; text-decoration:none; color:#000000; padding:0.2em 0.6em; font-size: large; } \n"
	,"#linkbar a:link { float:left; width:8em; font-weight: bold; text-decoration:none; color:#0000FF; padding:0.2em 0.6em; font-size: large; } \n"
	,"#linkbar a:active { float:left; width:8em; font-weight: bold; text-decoration:none; color:#00FF00; padding:0.2em 0.6em; font-size: large; } \n"
	,"#linkbar a:visited { float:left; width:8em; font-weight: bold; text-decoration:none; color:#000077; padding:0.2em 0.6em; font-size: large; } \n"
	,"#linkbar a:hover { font-weight: bold; text-decoration:underline; color:#777777; padding:0.2em 0.6em; font-size: large; } \n"
	,"#linkbar .nolink { float:left; width:8em; font-weight: bold; text-decoration:underline; text-decoration:underline; color:#000000; padding:0.2em 0.6em; font-size: large; } \n"
	,"#linkbar li {display:inlinefont-size: large; } \n"

	,"#pagemenu ul { float:left; width:90%; padding:0; margin:0; list-style-type:none; font-size: large; } \n"
	,"#pagemenu a { float:left; width:16em; font-weight: bold; text-decoration:none; color:#000000; padding:0.2em 0.6em; font-size: large; } \n"
	,"#pagemenu a:link { float:left; width:16em; font-weight: bold; text-decoration:none; color:#0000FF; padding:0.2em 0.6em; font-size: large; } \n"
	,"#pagemenu a:active { float:left; width:16em; font-weight: bold; text-decoration:none; color:#00FF00; padding:0.2em 0.6em; font-size: large; } \n"
	,"#pagemenu a:visited { float:left; width:16em; font-weight: bold; text-decoration:none; color:#000077; padding:0.2em 0.6em; font-size: large; } \n"
	,"#pagemenu a:hover { font-weight: bold; text-decoration:underline; color:#777777; padding:0.2em 0.6em; font-size: large; } \n"
	,"#pagemenu .nolink { float:left; width:16em; font-weight: bold; text-decoration:underline; text-decoration:underline; color:#000000; padding:0.2em 0.6em; font-size: large; } \n"
	,"#pagemenu li {display:inlinefont-size: large; } \n"


	,"#footer ul { float:left; width:100%; padding:0; margin:0; list-style-type:none; } \n"
	,"#footer a { float:left; width:8em; font-weight: bold; text-decoration:none; color:#000000; padding:0.2em 0.6em; } \n"
	,"#footer a:link { float:left; width:8em; font-weight: bold; text-decoration:none; color:#0000FF; padding:0.2em 0.6em; } \n"
	,"#footer a:active { float:left; width:8em; font-weight: bold; text-decoration:none; color:#00FF00; padding:0.2em 0.6em; } \n"
	,"#footer a:visited { float:left; width:8em; font-weight: bold; text-decoration:none; color:#000077; padding:0.2em 0.6em; } \n"
	,"#footer a:hover { font-weight: bold; width:8em; text-decoration:underline; color:#777777; padding:0.2em 0.6em; } \n"
	,"#footer .nolink1 { float:left; width:12em; font-weight: bold; text-decoration: none; color:#000000; padding:0.2em 0.6em;  text-align: right; } \n"
	,"#footer .nolink2 { float:left; width:18em; font-weight: bold; text-decoration: none; color:#000000; padding:0.2em 0.6em;  text-align: center; } \n"
	,"#footer .nolink3 { float:left; width:18em; font-weight: bold; text-decoration: none; color:#000000; padding:0.2em 0.6em;  text-align: right; } \n"
	,"#footer li {display:inline} \n"
	," -->","\n"
	,"</style>","\n"
	,"</head>","\n"
	,"<body>","\n"
	,"<div id=\"intsituteheader\">","\n"
	,"<a name=\"top\">&nbsp</a>","\n"
	,"<img src=\""
	, ppssumm_internal::pagename("logofile")
	,"\" alt=\"XMM Survey Science Centre Logo\" />","\n"
	,"<hr />"
	,"</div>","\n"
	,"END"
    };

    std::string* w = tmp_header;
    while (*w != "END")
    {
	_header.push_back(*w);
	w++;
    }

    std::string tmp_footer[] = 
    {
	"<div id=\"footer\">","\n"
	,"<ul>","\n"
	,"<li style=\"width: 8; text-align: left;\">"
	,"<a href=\"#top\">Return to top.</a>"
	,"</li>"
	,"<li class=\"nolink1\">"
	,_pc->taskversion()
	,"</li>"
	,"<li class=\"nolink2\">"
	,"Processing Date:&nbsp;"
	,_pc->procdate()
	,"</li>"
	,"<li class=\"nolink3\">"
	,"Odf Id:&nbsp;"
	,_pc->rev() , "_" , _pc->obs_id() , "_" , _pc->odf_version()
	,"</li>"
	,"</ul>","\n"
	,"</div>","\n"
	,"</body>","\n"
	,"END"
    };

    w = tmp_footer;
    while (*w != "END")
    {
	_footer.push_back(*w);
	w++;
    }

    std::string tmp_subtitle[] = 
    {
	"<div id=\"subtitle\">","\n"
	"<h1>"
	, _pc->obs_id()
	,"&nbsp;"
	,":PAGE_TITLE:"
	,"</h1>","\n"
	,"<table>","\n"
	,"<tr align=\"left\"><th>Revolution</th><th>Target</th><th>Slew Duration</th><th>Observer</th></tr>","\n"
	,"<tr align=\"left\">","\n"
	,"<td>",_pc->rev(),"</td>"
	,"<td>SLEW</td>"
	,"<td>",_pc->sched(),"</td>"
	,"<td>XMM-Newton</td>","\n"
	,"</tr>","\n"
	,"</table>","\n"
	,"<hr />","\n"
	,"</div>","\n"
	,"END"
    };

    w = tmp_subtitle;
    while (*w != "END")
    {
	_subtitle.push_back(*w);
	w++;
    }

    // Create a map of page_id -> page_title
    std::string _page_title_init[] = 
    {
	 "obspage" , "Observation Data Summary File"
	,"ppspage" , "PPS Processing Summary"
	, "epxpage" , "EPIC Processing Summary" 
	, "omxpage" , "OM Processing Summary" 
	, "rgxpage" , "RGS Processing Summary"
	, "catpage" , "ACDS Cross-Correlation Page"
	, "END"
    };
    w = _page_title_init;
    while ( *w != "END" )
    {
	_page_title[*w] = *(w+1);
	w+=2;
    }

   // Create a map of page_id -> page_title
    std::string _linkbar_text_init[] = 
    {
	 "obspage" , "OBS Summary"
	,"ppspage" , "PPS Summary"
	, "epxpage" , "EPIC Summary" 
	, "omxpage" , "OM Summary" 
	, "rgxpage" , "RGS Summary"
	, "catpage" , "Catalogue Summary"
	, "END"
    };
    w = _linkbar_text_init;
    while ( *w != "END" )
    {
	_linkbar_text[*w] = *(w+1);
	w+=2;
    }


}

void HtmlPage::subtitle( ostream& os ) const 
{
    for (stringlist_ci CI = _subtitle.begin()
	; CI != _subtitle.end()
	; CI++
    ){
	if (*CI != ":PAGE_TITLE:") os << *CI;
	else os << page_title( _page_id ) ;
    }
}

void HtmlPage::linkbar( ostream& os ) const
{
    os  << "<div id=\"linkbar\">" << "\n";

    stringlist pages = _pc->page_id();
    os << "<ul>" << "\n";
    for(stringlist_ci CI = pages.begin() ; CI != pages.end() ; ++CI )
    {
        if( *CI == _page_id ) 
        {
	   //os << page_title() << "\n";
	   os << "<li class=\"nolink\">" << linkbar_text( *CI ) << "</li>" << "\n";
        }
        else 
        {
	    os  << "<li><a href=\""
		<< page_filename( *CI )
		<< "\">"
		<< linkbar_text( *CI )
		<< "</a></li>"
		<< "\n"
	    ;
        }
    }
    os << "</ul>" << "\n";
    os << "<br clear=\"left\"/>" << "\n";

    os  << "<hr />"
	<< "\n"
	<< "</div>"
	<< "\n"
    ;
}

void HtmlPage::start( ostream& os ) const
{
    for (stringlist_ci CI = _header.begin()
	; CI != _header.end()
	; CI++
    ){
	if (*CI != ":PAGE_TITLE:") os << *CI;
	else os << page_title( _page_id );
    }

    linkbar( os );
    subtitle( os );
    
}

// Write out the bottom of an html page
void HtmlPage::end( ostream& os ) const
{
    for (stringlist_ci CI = _footer.begin()
	; CI != _footer.end()
	; CI++
    ){
	os << *CI;
    }
}


