// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMM_HTMLPAGE
#define PPSSUMM_HTMLPAGE

#include <string>
#include <algorithm>
#include <functional>
#include <iterator>

#include "ppssumm_typedef.h"
#include "PpssummCommon.h"

#include "ExtendFitsColumn.h"
#include "string_algo.h"
#include "stripws.h"
#include "STime.h"
#include "task_info.h"

#include "param.h"
#include "Tracer.h"
#include "errstr.h"

/*
 * Alternate init_
 * "<?xml version=\"1.0\" encoding=\"UTF-8\"?> <!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\">"
 *
 */
class HtmlPage 
{

    /* WARNING
     * title_ is what goes in the <title> tags.
     */
 
    public:
    HtmlPage(const PpssummCommon* pc ); 
    ~HtmlPage() { _pc = NULL; }

    std::string page_filename( std::string s="" ) const
    {
	if (! s.length() ) s = _page_id;
	return ppssumm_internal::pagename(s);
    }
    std::string page_title( std::string s="" ) const
    {
	if (! s.length() ) s = _page_id;
	ssmap_ci CI = _page_title.find( s );
	return (CI != _page_title.end() ? CI->second : "THIS IS NOT RIGHT");
    }
    // Create the linkbar using current _page_id.
    std::string linkbar_text( std::string s="" ) const
    {
	if (! s.length() ) s = _page_id;
	ssmap_ci CI = _linkbar_text.find( s );
	return (CI != _linkbar_text.end() ? CI->second : "THIS IS NOT RIGHT");
    }
    void linkbar( ostream& ) const;
    // Create the subtitle using current _page_id.
    void subtitle( ostream& ) const;
    
    // Write out the top of an html page 
    void start( ostream& ) const;
    
    // Write out the bottom of an html page
    void end( ostream& ) const;

    // Getters
    std::string page_id() const { return _page_id; }

    // Setters
    void page_id( const std::string s ) { _page_id = s; }

 
    private:
    const PpssummCommon* _pc;
    stringlist _header;
    stringlist _footer;
    stringlist _subtitle;
    std::string _page_id;     
    ssmap _page_title;
    ssmap _linkbar_text;
};

#endif
