// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "Pindex.h"

std::string CatcatRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _catname;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Catcat::Catcat( Table* t )
: std::vector<CatcatRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int real32_num=1;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=15;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "CONTENT"
        , "CATNAME"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( CatcatRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[4]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
        ));
    }
}

std::string Catcat::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Catcat\n";
    for ( Catcat::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string CatobsRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Catobs::Catobs( Table* t )
: std::vector<CatobsRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int real32_num=1;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=4;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=14;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "CONTENT"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( CatobsRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[4]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
        ));
    }
}
std::string Catobs::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Catobs\n";
    for ( Catobs::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string CatsrcRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _plate_ra;
    rtn << "|" << _plate_dec;
    rtn << "|" << _plate_filter;
    rtn << "|" << _plate_id;
    rtn << "|" << _plate_origin;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Catsrc::Catsrc( Table* t )
: std::vector<CatsrcRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int real32_num=3;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
        , "PLATE_RA"
        , "PLATE_DEC"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=17;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "CONTENT"
        , "PLATE_FILTER"
        , "PLATE_ID"
        , "PLATE_ORIGIN"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( CatsrcRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[4]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(real32_ci[1]+i)
            , *(real32_ci[2]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
            , *(string_ci[15]+i)
            , *(string_ci[16]+i)
        ));
    }
}
std::string Catsrc::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Catsrc\n";
    for ( Catsrc::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

//#

std::string EpicexpRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _exp_id;
    rtn << "|" << _mode;
    rtn << "|" << _datamode;
    rtn << "|" << _ep_filter;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _exp_start;
    rtn << "|" << _exp_stop;
    rtn << "|" << _exp_duration;
    rtn << "|" << _band;
    rtn << "|" << _exposure;
    rtn << "|" << _srcdet;
    rtn << "|" << _ssp;
    rtn << "|" << _flarescreened;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Epicexp::Epicexp( Table* t )
: std::vector<EpicexpRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int bool8_num=3;
    bool8list bool8_cd[bool8_num];
    bool8list_ci bool8_ci[bool8_num];
    std::string bool8_colname[bool8_num] = 
    {
        "SRCDET"
        , "SSP"
        , "FLARESCREENED"
    };
    for(unsigned int i=0 ; i < bool8_num; i++)
    {
        bool8_cd[i] = bool8list();
        bool8Column( t , bool8_colname[i] , back_inserter( bool8_cd[i] ) );
        bool8_ci[i] = bool8_cd[i].begin();
    }
    
    const unsigned int int32_num=1;
    int32list int32_cd[int32_num];
    int32list_ci int32_ci[int32_num];
    std::string int32_colname[int32_num] = 
    {
        "BAND"
    };
    for(unsigned int i=0 ; i < int32_num; i++)
    {
        int32_cd[i] = int32list();
        int32Column( t , int32_colname[i] , back_inserter( int32_cd[i] ) );
        int32_ci[i] = int32_cd[i].begin();
    }
    
    const unsigned int real32_num=3;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
        , "EXP_DURATION"
        , "EXPOSURE"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=21;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "EXP_ID"
        , "MODE"
        , "DATAMODE"
        , "EP_FILTER"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "CONTENT"
        , "EXP_START"
        , "EXP_STOP"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( EpicexpRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[9]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(real32_ci[1]+i)
            , *(int32_ci[0]+i)
            , *(real32_ci[2]+i)
            , *(bool8_ci[0]+i)
            , *(bool8_ci[1]+i)
            , *(bool8_ci[2]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
            , *(string_ci[15]+i)
            , *(string_ci[16]+i)
            , *(string_ci[17]+i)
            , *(string_ci[18]+i)
            , *(string_ci[19]+i)
            , *(string_ci[20]+i)
        ));
    }
}
std::string Epicexp::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Epicexp\n";
    for ( Epicexp::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string EpicobsRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Epicobs::Epicobs( Table* t )
: std::vector<EpicobsRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int real32_num=1;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=15;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "CONTENT"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( EpicobsRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[5]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
        ));
    }
}
std::string Epicobs::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Epicobs\n";
    for ( Epicobs::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string EpicsrcRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _exp_id;
    rtn << "|" << _src_num;
    rtn << "|" << _band;
    rtn << "|" << _mode;
    rtn << "|" << _datamode;
    rtn << "|" << _ep_filter;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _exp_start;
    rtn << "|" << _exp_stop;
    rtn << "|" << _exp_duration;
    rtn << "|" << _ra_obj;
    rtn << "|" << _dec_obj;
    rtn << "|" << _radec_err;
    rtn << "|" << _lii_obj;
    rtn << "|" << _bii_obj;
    rtn << "|" << _pn_cts;
    rtn << "|" << _pn_cts_err;
    rtn << "|" << _m1_cts;
    rtn << "|" << _m1_cts_err;
    rtn << "|" << _m2_cts;
    rtn << "|" << _m2_cts_err;
    rtn << "|" << _pn_tot_flux;
    rtn << "|" << _pn_tot_flux_err;
    rtn << "|" << _m1_tot_flux;
    rtn << "|" << _m1_tot_flux_err;
    rtn << "|" << _m2_tot_flux;
    rtn << "|" << _m2_tot_flux_err;
    rtn << "|" << _ep_tot_flux;
    rtn << "|" << _ep_tot_flux_err;
    rtn << "|" << _pn_tot;
    rtn << "|" << _pn_tot_err;
    rtn << "|" << _m1_tot;
    rtn << "|" << _m1_tot_err;
    rtn << "|" << _m2_tot;
    rtn << "|" << _m2_tot_err;
    rtn << "|" << _ep_tot;
    rtn << "|" << _ep_tot_err;
    rtn << "|" << _pn_det_ml;
    rtn << "|" << _m1_det_ml;
    rtn << "|" << _m2_det_ml;
    rtn << "|" << _ep_det_ml;
    rtn << "|" << _ep_extent;
    rtn << "|" << _ep_ext_err;
    rtn << "|" << _pn_ontime;
    rtn << "|" << _m1_ontime;
    rtn << "|" << _m2_ontime;
    rtn << "|" << _ep_hr1;
    rtn << "|" << _ep_hr1_err;
    rtn << "|" << _ep_hr2;
    rtn << "|" << _ep_hr2_err;
    rtn << "|" << _ep_hr3;
    rtn << "|" << _ep_hr3_err;
    rtn << "|" << _ep_hr4;
    rtn << "|" << _ep_hr4_err;
    rtn << "|" << _srcdet;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _ep_flag;
    rtn << "|" << _m1_flag;
    rtn << "|" << _m2_flag;
    rtn << "|" << _pn_flag;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Epicsrc::Epicsrc( Table* t )
: std::vector<EpicsrcRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int bool8_num=1;
    bool8list bool8_cd[bool8_num];
    bool8list_ci bool8_ci[bool8_num];
    std::string bool8_colname[bool8_num] = 
    {
        "SRCDET"
    };
    for(unsigned int i=0 ; i < bool8_num; i++)
    {
        bool8_cd[i] = bool8list();
        bool8Column( t , bool8_colname[i] , back_inserter( bool8_cd[i] ) );
        bool8_ci[i] = bool8_cd[i].begin();
    }
    
    const unsigned int int16_num=1;
    int16list int16_cd[int16_num];
    int16list_ci int16_ci[int16_num];
    std::string int16_colname[int16_num] = 
    {
        "SRC_NUM"
    };
    for(unsigned int i=0 ; i < int16_num; i++)
    {
        int16_cd[i] = int16list();
        int16Column( t , int16_colname[i] , back_inserter( int16_cd[i] ) );
        int16_ci[i] = int16_cd[i].begin();
    }
    
    const unsigned int int32_num=1;
    int32list int32_cd[int32_num];
    int32list_ci int32_ci[int32_num];
    std::string int32_colname[int32_num] = 
    {
        "BAND"
    };
    for(unsigned int i=0 ; i < int32_num; i++)
    {
        int32_cd[i] = int32list();
        int32Column( t , int32_colname[i] , back_inserter( int32_cd[i] ) );
        int32_ci[i] = int32_cd[i].begin();
    }
    
    const unsigned int real32_num=42;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
        , "EXP_DURATION"
        , "RADEC_ERR"
        , "PN_CTS"
        , "PN_CTS_ERR"
        , "M1_CTS"
        , "M1_CTS_ERR"
        , "M2_CTS"
        , "M2_CTS_ERR"
        , "PN_TOT_FLUX"
        , "PN_TOT_FLUX_ERR"
        , "M1_TOT_FLUX"
        , "M1_TOT_FLUX_ERR"
        , "M2_TOT_FLUX"
        , "M2_TOT_FLUX_ERR"
        , "EP_TOT_FLUX"
        , "EP_TOT_FLUX_ERR"
        , "PN_TOT"
        , "PN_TOT_ERR"
        , "M1_TOT"
        , "M1_TOT_ERR"
        , "M2_TOT"
        , "M2_TOT_ERR"
        , "EP_TOT"
        , "EP_TOT_ERR"
        , "PN_DET_ML"
        , "M1_DET_ML"
        , "M2_DET_ML"
        , "EP_DET_ML"
        , "EP_EXTENT"
        , "EP_EXT_ERR"
        , "PN_ONTIME"
        , "M1_ONTIME"
        , "M2_ONTIME"
        , "EP_HR1"
        , "EP_HR1_ERR"
        , "EP_HR2"
        , "EP_HR2_ERR"
        , "EP_HR3"
        , "EP_HR3_ERR"
        , "EP_HR4"
        , "EP_HR4_ERR"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=9;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
        , "RA_OBJ"
        , "DEC_OBJ"
        , "LII_OBJ"
        , "BII_OBJ"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=25;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "EXP_ID"
        , "MODE"
        , "DATAMODE"
        , "EP_FILTER"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "CONTENT"
        , "EXP_START"
        , "EXP_STOP"
        , "OBSERVER"
        , "OBJECT"
        , "EP_FLAG"
        , "M1_FLAG"
        , "M2_FLAG"
        , "PN_FLAG"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
	// remember to convert RADEC_ERR from arcseconds to degrees

        this->push_back( EpicsrcRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(int16_ci[0]+i)
            , *(int32_ci[0]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[9]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(real32_ci[1]+i)
            , *(real64_ci[5]+i)
            , *(real64_ci[6]+i)
            , *(real32_ci[2]+i)/3600.0
            , *(real64_ci[7]+i)
            , *(real64_ci[8]+i)
            , *(real32_ci[3]+i)
            , *(real32_ci[4]+i)
            , *(real32_ci[5]+i)
            , *(real32_ci[6]+i)
            , *(real32_ci[7]+i)
            , *(real32_ci[8]+i)
            , *(real32_ci[9]+i)
            , *(real32_ci[10]+i)
            , *(real32_ci[11]+i)
            , *(real32_ci[12]+i)
            , *(real32_ci[13]+i)
            , *(real32_ci[14]+i)
            , *(real32_ci[15]+i)
            , *(real32_ci[16]+i)
            , *(real32_ci[17]+i)
            , *(real32_ci[18]+i)
            , *(real32_ci[19]+i)
            , *(real32_ci[20]+i)
            , *(real32_ci[21]+i)
            , *(real32_ci[22]+i)
            , *(real32_ci[23]+i)
            , *(real32_ci[24]+i)
            , *(real32_ci[25]+i)
            , *(real32_ci[26]+i)
            , *(real32_ci[27]+i)
            , *(real32_ci[28]+i)
            , *(real32_ci[29]+i)
            , *(real32_ci[30]+i)
            , *(real32_ci[31]+i)
            , *(real32_ci[32]+i)
            , *(real32_ci[33]+i)
            , *(real32_ci[34]+i)
            , *(real32_ci[35]+i)
            , *(real32_ci[36]+i)
            , *(real32_ci[37]+i)
            , *(real32_ci[38]+i)
            , *(real32_ci[39]+i)
            , *(real32_ci[40]+i)
            , *(real32_ci[41]+i)
            , *(bool8_ci[0]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
            , *(string_ci[15]+i)
            , *(string_ci[16]+i)
            , *(string_ci[17]+i)
            , *(string_ci[18]+i)
            , *(string_ci[19]+i)
            , *(string_ci[20]+i)
            , *(string_ci[21]+i)
            , *(string_ci[22]+i)
            , *(string_ci[23]+i)
            , *(string_ci[24]+i)
        ));
    }
}

std::string Epicsrc::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Epicsrc\n";
    for ( Epicsrc::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}



//#


std::string InstsummRow::print() const
{
    ppssumm_internal::toString<bool8> tostring;
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << tostring(_active);
    rtn << "|" << _nexposures;
    rtn << "|" << _priority;
    rtn << "|\n";
    return rtn.str();
}

Instsumm::Instsumm( Table* t )
: std::vector<InstsummRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int bool8_num=1;
    bool8list bool8_cd[bool8_num];
    bool8list_ci bool8_ci[bool8_num];
    std::string bool8_colname[bool8_num] = 
    {
        "ACTIVE"
    };
    for(unsigned int i=0 ; i < bool8_num; i++)
    {
        bool8_cd[i] = bool8list();
        bool8Column( t , bool8_colname[i] , back_inserter( bool8_cd[i] ) );
        bool8_ci[i] = bool8_cd[i].begin();
    }
    
    const unsigned int int32_num=2;
    int32list int32_cd[int32_num];
    int32list_ci int32_ci[int32_num];
    std::string int32_colname[int32_num] = 
    {
        "NEXPOSURES"
        , "PRIORITY"
    };
    for(unsigned int i=0 ; i < int32_num; i++)
    {
        int32_cd[i] = int32list();
        int32Column( t , int32_colname[i] , back_inserter( int32_cd[i] ) );
        int32_ci[i] = int32_cd[i].begin();
    }
    
    const unsigned int string_num=1;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( InstsummRow( i+1 
            , *(string_ci[0]+i)
            , *(bool8_ci[0]+i)
            , *(int32_ci[0]+i)
            , *(int32_ci[1]+i)
        ));
    }
}
std::string Instsumm::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Instsumm\n";
    for ( Instsumm::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string OmexpRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _detector;
    rtn << "|" << _om_filter;
    rtn << "|" << _mode;
    rtn << "|" << _datamode;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _exp_id;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _exp_start;
    rtn << "|" << _exp_stop;
    rtn << "|" << _exp_duration;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Omexp::Omexp( Table* t )
: std::vector<OmexpRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int real32_num=2;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
        , "EXP_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=22;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "DETECTOR"
        , "OM_FILTER"
        , "MODE"
        , "DATAMODE"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "EXP_ID"
        , "CONTENT"
        , "EXP_START"
        , "EXP_STOP"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( OmexpRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(real32_ci[1]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
            , *(string_ci[15]+i)
            , *(string_ci[16]+i)
            , *(string_ci[17]+i)
            , *(string_ci[18]+i)
            , *(string_ci[19]+i)
            , *(string_ci[20]+i)
            , *(string_ci[21]+i)
        ));
    }
}

std::string Omexp::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Omexp\n";
    for ( Omexp::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string OmobsRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _detector;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}


Omobs::Omobs( Table* t )
: std::vector<OmobsRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int real32_num=1;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=16;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "DETECTOR"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "CONTENT"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( OmobsRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(string_ci[5]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[6]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
            , *(string_ci[15]+i)
        ));
    }
}
std::string Omobs::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Omobs\n";
    for ( Omobs::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}


std::string OmsrcRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _detector;
    rtn << "|" << _datamode;
    rtn << "|" << _mode;
    rtn << "|" << _om_filter;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _exp_id;
    rtn << "|" << _osw_id;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _exp_start;
    rtn << "|" << _exp_stop;
    rtn << "|" << _exp_duration;
    rtn << "|" << _src_num;
    rtn << "|" << _ra_obj;
    rtn << "|" << _dec_obj;
    rtn << "|" << _lii_obj;
    rtn << "|" << _bii_obj;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Omsrc::Omsrc( Table* t )
: std::vector<OmsrcRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int int16_num=1;
    int16list int16_cd[int16_num];
    int16list_ci int16_ci[int16_num];
    std::string int16_colname[int16_num] = 
    {
        "SRC_NUM"
    };
    for(unsigned int i=0 ; i < int16_num; i++)
    {
        int16_cd[i] = int16list();
        int16Column( t , int16_colname[i] , back_inserter( int16_cd[i] ) );
        int16_ci[i] = int16_cd[i].begin();
    }
    
    const unsigned int real32_num=2;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
        , "EXP_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=9;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
        , "RA_OBJ"
        , "DEC_OBJ"
        , "LII_OBJ"
        , "BII_OBJ"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=23;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "DETECTOR"
        , "DATAMODE"
        , "MODE"
        , "OM_FILTER"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "EXP_ID"
        , "OSW_ID"
        , "CONTENT"
        , "EXP_START"
        , "EXP_STOP"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( OmsrcRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(real32_ci[1]+i)
            , *(int16_ci[0]+i)
            , *(real64_ci[5]+i)
            , *(real64_ci[6]+i)
            , *(real64_ci[7]+i)
            , *(real64_ci[8]+i)
            , *(string_ci[14]+i)
            , *(string_ci[15]+i)
            , *(string_ci[16]+i)
            , *(string_ci[17]+i)
            , *(string_ci[18]+i)
            , *(string_ci[19]+i)
            , *(string_ci[20]+i)
            , *(string_ci[21]+i)
            , *(string_ci[22]+i)
        ));
    }
}
std::string Omsrc::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Omsrc\n";
    for ( Omsrc::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string OmswRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _detector;
    rtn << "|" << _om_filter;
    rtn << "|" << _mode;
    rtn << "|" << _datamode;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _exp_id;
    rtn << "|" << _osw_id;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _exp_start;
    rtn << "|" << _exp_stop;
    rtn << "|" << _exp_duration;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}


std::string PpsobsRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Omsw::Omsw( Table* t )
: std::vector<OmswRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int real32_num=2;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
        , "EXP_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=23;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "DETECTOR"
        , "OM_FILTER"
        , "MODE"
        , "DATAMODE"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "EXP_ID"
        , "OSW_ID"
        , "CONTENT"
        , "EXP_START"
        , "EXP_STOP"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( OmswRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(real32_ci[1]+i)
            , *(string_ci[14]+i)
            , *(string_ci[15]+i)
            , *(string_ci[16]+i)
            , *(string_ci[17]+i)
            , *(string_ci[18]+i)
            , *(string_ci[19]+i)
            , *(string_ci[20]+i)
            , *(string_ci[21]+i)
            , *(string_ci[22]+i)
        ));
    }
}
std::string Omsw::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Omsw\n";
    for ( Omsw::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}


Ppsobs::Ppsobs( Table* t )
: std::vector<PpsobsRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int real32_num=1;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=14;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "CONTENT"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( PpsobsRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[4]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
        ));
    }
}
std::string Ppsobs::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Ppsobs\n";
    for ( Ppsobs::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string RgsexpRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _mode;
    rtn << "|" << _datamode;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _exp_id;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _exp_start;
    rtn << "|" << _exp_stop;
    rtn << "|" << _exp_duration;
    rtn << "|" << _filtgrps;
    rtn << "|" << _ocb;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Rgsexp::Rgsexp( Table* t )
: std::vector<RgsexpRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int int16_num=1;
    int16list int16_cd[int16_num];
    int16list_ci int16_ci[int16_num];
    std::string int16_colname[int16_num] = 
    {
        "OCB"
    };
    for(unsigned int i=0 ; i < int16_num; i++)
    {
        int16_cd[i] = int16list();
        int16Column( t , int16_colname[i] , back_inserter( int16_cd[i] ) );
        int16_ci[i] = int16_cd[i].begin();
    }
   
    const unsigned int int32_num=1;
    int32list int32_cd[int32_num];
    int32list_ci int32_ci[int32_num];
    std::string int32_colname[int32_num] = 
    {
        "FILTGRPS"
    };
    for(unsigned int i=0 ; i < int32_num; i++)
    {
        int32_cd[i] = int32list();
        int32Column( t , int32_colname[i] , back_inserter( int32_cd[i] ) );
        int32_ci[i] = int32_cd[i].begin();
    }
 
    const unsigned int real32_num=2;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
        , "EXP_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=20;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "MODE"
        , "DATAMODE"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "EXP_ID"
        , "CONTENT"
        , "EXP_START"
        , "EXP_STOP"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( RgsexpRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(real32_ci[1]+i)
	    , *(int32_ci[0]+i)
            , *(int16_ci[0]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
            , *(string_ci[15]+i)
            , *(string_ci[16]+i)
            , *(string_ci[17]+i)
            , *(string_ci[18]+i)
            , *(string_ci[19]+i)
        ));
    }
}

std::string Rgsexp::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Rgsexp\n";
    for ( Rgsexp::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string RgsobsRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Rgsobs::Rgsobs( Table* t )
: std::vector<RgsobsRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int real32_num=1;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=5;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=15;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "CONTENT"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( RgsobsRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[5]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[6]+i)
            , *(string_ci[7]+i)
            , *(string_ci[8]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
        ));
    }
}
std::string Rgsobs::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Rgsobs\n";
    for ( Rgsobs::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}

std::string RgssrcRow::print() const
{
    std::stringstream rtn; 
    rtn << "|" << _instrument;
    rtn << "|" << _mode;
    rtn << "|" << _datamode;
    rtn << "|" << _obs_mode;
    rtn << "|" << _obs_id;
    rtn << "|" << _obs_start;
    rtn << "|" << _obs_stop;
    rtn << "|" << _obs_duration;
    rtn << "|" << _exp_id;
    rtn << "|" << _ocb;
    rtn << "|" << _content;
    rtn << "|" << _ra_pnt;
    rtn << "|" << _dec_pnt;
    rtn << "|" << _pa_pnt;
    rtn << "|" << _lii_pnt;
    rtn << "|" << _bii_pnt;
    rtn << "|" << _exp_start;
    rtn << "|" << _exp_stop;
    rtn << "|" << _exp_duration;
    rtn << "|" << _filtgrps;
    rtn << "|" << _rgsorder;
    rtn << "|" << _src_num;
    rtn << "|" << _ra_obj;
    rtn << "|" << _dec_obj;
    rtn << "|" << _lii_obj;
    rtn << "|" << _bii_obj;
    rtn << "|" << _ep_tot;
    rtn << "|" << _ep_tot_err;
    rtn << "|" << _observer;
    rtn << "|" << _object;
    rtn << "|" << _vvflags;
    rtn << "|" << _procrevision;
    rtn << "|" << _procdate;
    rtn << "|" << _sasversion;
    rtn << "|" << _ppsversion;
    rtn << "|" << _format;
    rtn << "|" << _name;
    rtn << "|\n";
    return rtn.str();
}

Rgssrc::Rgssrc( Table* t )
: std::vector<RgssrcRow>() 
{
    unsigned long rows = t->rows();
    const unsigned int int16_num=3;
    int16list int16_cd[int16_num];
    int16list_ci int16_ci[int16_num];
    std::string int16_colname[int16_num] = 
    {
        "OCB"
        , "RGSORDER"
        , "SRC_NUM"
    };
    for(unsigned int i=0 ; i < int16_num; i++)
    {
        int16_cd[i] = int16list();
        int16Column( t , int16_colname[i] , back_inserter( int16_cd[i] ) );
        int16_ci[i] = int16_cd[i].begin();
    }
  
    const unsigned int int32_num=1;
    int32list int32_cd[int32_num];
    int32list_ci int32_ci[int32_num];
    std::string int32_colname[int32_num] = 
    {
        "FILTGRPS"
    };
    for(unsigned int i=0 ; i < int32_num; i++)
    {
        int32_cd[i] = int32list();
        int32Column( t , int32_colname[i] , back_inserter( int32_cd[i] ) );
        int32_ci[i] = int32_cd[i].begin();
    }
  
    const unsigned int real32_num=4;
    real32list real32_cd[real32_num];
    real32list_ci real32_ci[real32_num];
    std::string real32_colname[real32_num] = 
    {
        "OBS_DURATION"
        , "EXP_DURATION"
        , "EP_TOT"
        , "EP_TOT_ERR"
    };
    for(unsigned int i=0 ; i < real32_num; i++)
    {
        real32_cd[i] = real32list();
        real32Column( t , real32_colname[i] , back_inserter( real32_cd[i] ) );
        real32_ci[i] = real32_cd[i].begin();
    }
    
    const unsigned int real64_num=9;
    real64list real64_cd[real64_num];
    real64list_ci real64_ci[real64_num];
    std::string real64_colname[real64_num] = 
    {
        "RA_PNT"
        , "DEC_PNT"
        , "PA_PNT"
        , "LII_PNT"
        , "BII_PNT"
        , "RA_OBJ"
        , "DEC_OBJ"
        , "LII_OBJ"
        , "BII_OBJ"
    };
    for(unsigned int i=0 ; i < real64_num; i++)
    {
        real64_cd[i] = real64list();
        real64Column( t , real64_colname[i] , back_inserter( real64_cd[i] ) );
        real64_ci[i] = real64_cd[i].begin();
    }
    
    const unsigned int string_num=20;
    stringlist string_cd[string_num];
    stringlist_ci string_ci[string_num];
    std::string string_colname[string_num] = 
    {
        "INSTRUMENT"
        , "MODE"
        , "DATAMODE"
        , "OBS_MODE"
        , "OBS_ID"
        , "OBS_START"
        , "OBS_STOP"
        , "EXP_ID"
        , "CONTENT"
        , "EXP_START"
        , "EXP_STOP"
        , "OBSERVER"
        , "OBJECT"
        , "VVFLAGS"
        , "PROCREVISION"
        , "PROCDATE"
        , "SASVERSION"
        , "PPSVERSION"
        , "FORMAT"
        , "NAME"
    };
    for(unsigned int i=0 ; i < string_num; i++)
    {
        string_cd[i] = stringlist();
        stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
        string_ci[i] = string_cd[i].begin();
    }
    
    for(unsigned long i=0; i < rows ; i++)
    {
        this->push_back( RgssrcRow( i+1 
            , *(string_ci[0]+i)
            , *(string_ci[1]+i)
            , *(string_ci[2]+i)
            , *(string_ci[3]+i)
            , *(string_ci[4]+i)
            , *(string_ci[5]+i)
            , *(string_ci[6]+i)
            , *(real32_ci[0]+i)
            , *(string_ci[7]+i)
            , *(int16_ci[0]+i)
            , *(string_ci[8]+i)
            , *(real64_ci[0]+i)
            , *(real64_ci[1]+i)
            , *(real64_ci[2]+i)
            , *(real64_ci[3]+i)
            , *(real64_ci[4]+i)
            , *(string_ci[9]+i)
            , *(string_ci[10]+i)
            , *(real32_ci[1]+i)
            , *(int32_ci[0]+i)
            , *(int16_ci[1]+i)
            , *(int16_ci[2]+i)
            , *(real64_ci[5]+i)
            , *(real64_ci[6]+i)
            , *(real64_ci[7]+i)
            , *(real64_ci[8]+i)
            , *(real32_ci[2]+i)
            , *(real32_ci[3]+i)
            , *(string_ci[11]+i)
            , *(string_ci[12]+i)
            , *(string_ci[13]+i)
            , *(string_ci[14]+i)
            , *(string_ci[15]+i)
            , *(string_ci[16]+i)
            , *(string_ci[17]+i)
            , *(string_ci[18]+i)
            , *(string_ci[19]+i)
        ));
    }
}
std::string Rgssrc::print() const
{
    std::stringstream rtn;
    rtn << "Table:  Rgssrc\n";
    for ( Rgssrc::const_iterator CI = this->begin() ; CI != this->end() ; CI++ )
    {
        rtn << CI->print();
    }
    return rtn.str();
}


Pindex::Pindex( const std::string& filename ) 
: _filename(filename) , 
_catcat()
, _catobs()
, _catsrc()
, _epicexp()
, _epicobs()
, _epicsrc()
, _instsumm()
, _omexp()
, _omobs()
, _omsrc()
, _omsw()
, _ppsobs()
, _rgsexp()
, _rgsobs()
, _rgssrc()
{
    DataSet* ds = dataSetServer->open( _filename , Dal::Read);
    Table* table;
    char* bn = "CATCAT";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _catcat = Catcat( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "CATOBS";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _catobs = Catobs( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "CATSRC";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _catsrc = Catsrc( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "EPICEXP";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _epicexp = Epicexp( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "EPICOBS";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _epicobs = Epicobs( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "EPICSRC";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _epicsrc = Epicsrc( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "INSTSUMM";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _instsumm = Instsumm( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "OMEXP";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );

        _omexp = Omexp( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "OMOBS";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _omobs = Omobs( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "OMSRC";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _omsrc = Omsrc( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "OMSW";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _omsw = Omsw( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "PPSOBS";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _ppsobs = Ppsobs( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "RGSEXP";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _rgsexp = Rgsexp( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "RGSOBS";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _rgsobs = Rgsobs( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    bn = "RGSSRC";
    if ( ds->hasBlock( bn ) )
    {
    errstr << "Pindex reading: " << bn << "." << Sas::message(Msg::App, Msg::Sparse);
        table = ds->table( bn );
        _rgssrc = Rgssrc( table );
    errstr << "Pindex reading: " << bn << " Done." << Sas::message(Msg::App, Msg::Sparse);
    }
    dataSetServer->close( ds );
}
std::string Pindex::print() const
{
    std::stringstream rtn;
    rtn <<  "Pindex: " << _filename;
    rtn <<  _catcat.print();
    rtn <<  _catobs.print();
    rtn <<  _catsrc.print();
    rtn <<  _epicexp.print();
    rtn <<  _epicobs.print();
    rtn <<  _epicsrc.print();
    rtn <<  _instsumm.print();
    rtn <<  _omexp.print();
    rtn <<  _omobs.print();
    rtn <<  _omsrc.print();
    rtn <<  _omsw.print();
    rtn <<  _ppsobs.print();
    rtn <<  _rgsexp.print();
    rtn <<  _rgsobs.print();
    rtn <<  _rgssrc.print();
    rtn <<  std::endl;
    return rtn.str();
}
