// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMM_PINDEX
#define PPSSUMM_PINDEX

#include <string>
#include <vector>
#include <algorithm>
#include <iterator>
#include <iostream>

// SAS 1
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"

// PPSSUMM Type classes
#include "Filename.h"
#include "Instrument.h"
#include "Src_num.h"
#include "Band.h"
#include "Rgsorder.h"
#include "Degrees.h"
#include "Datamode.h"
#include "Filtgrps.h"
#include "Pindex_typedef.h"

// PPSSUMM Extras

#include "ppssumm_typedef.h"
#include "ExtendFitsColumn.h"
#include "string_algo.h"

#ifndef PINDEX_CATCAT
#define PINDEX_CATCAT

class CatcatRow
{
    public:
    CatcatRow() {}
    CatcatRow(
        const unsigned int& r
        , const Obs_mode& v0
        , const Obs_id& v1
        , const Obs_start& v2
        , const Obs_stop& v3
        , const Obs_duration& v4
        , const Content& v5
        , const Ra_pnt& v6
        , const Dec_pnt& v7
        , const Pa_pnt& v8
        , const Lii_pnt& v9
        , const Bii_pnt& v10
        , const Catname& v11
        , const Observer& v12
        , const Object& v13
        , const Vvflags& v14
        , const Procrevision& v15
        , const Procdate& v16
        , const Sasversion& v17
        , const Ppsversion& v18
        , const Format& v19
        , const Name& v20
    ) : 
    _row(r)
    , _obs_mode(v0)
    , _obs_id(v1)
    , _obs_start(v2)
    , _obs_stop(v3)
    , _obs_duration(v4)
    , _content(v5)
    , _ra_pnt(v6)
    , _dec_pnt(v7)
    , _pa_pnt(v8)
    , _lii_pnt(v9)
    , _bii_pnt(v10)
    , _catname(v11)
    , _observer(v12)
    , _object(v13)
    , _vvflags(v14)
    , _procrevision(v15)
    , _procdate(v16)
    , _sasversion(v17)
    , _ppsversion(v18)
    , _format(v19)
    , _name(v20)
    { 
    }
    unsigned int row() const { return _row; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Catname catname() const { return _catname; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void catname(const Catname& v) { _catname = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Catname _catname;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Catcat
: public std::vector<CatcatRow>
{
    public:
    Catcat() : std::vector<CatcatRow>() {}
    Catcat( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_CATOBS
#define PINDEX_CATOBS

class CatobsRow
{
    public:
    CatobsRow() {}
    CatobsRow(
        const unsigned int& r
        , const Obs_mode& v0
        , const Obs_id& v1
        , const Obs_start& v2
        , const Obs_stop& v3
        , const Obs_duration& v4
        , const Content& v5
        , const Ra_pnt& v6
        , const Dec_pnt& v7
        , const Pa_pnt& v8
        , const Lii_pnt& v9
        , const Observer& v10
        , const Object& v11
        , const Vvflags& v12
        , const Procrevision& v13
        , const Procdate& v14
        , const Sasversion& v15
        , const Ppsversion& v16
        , const Format& v17
        , const Name& v18
    ) : 
    _row(r)
    , _obs_mode(v0)
    , _obs_id(v1)
    , _obs_start(v2)
    , _obs_stop(v3)
    , _obs_duration(v4)
    , _content(v5)
    , _ra_pnt(v6)
    , _dec_pnt(v7)
    , _pa_pnt(v8)
    , _lii_pnt(v9)
    , _observer(v10)
    , _object(v11)
    , _vvflags(v12)
    , _procrevision(v13)
    , _procdate(v14)
    , _sasversion(v15)
    , _ppsversion(v16)
    , _format(v17)
    , _name(v18)
    { 
    }
    unsigned int row() const { return _row; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Catobs
: public std::vector<CatobsRow>
{
    public:
    Catobs() : std::vector<CatobsRow>() {}
    Catobs( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_CATSRC
#define PINDEX_CATSRC

class CatsrcRow
{
    public:
    CatsrcRow() {}
    CatsrcRow(
        const unsigned int& r
        , const Obs_mode& v0
        , const Obs_id& v1
        , const Obs_start& v2
        , const Obs_stop& v3
        , const Obs_duration& v4
        , const Content& v5
        , const Ra_pnt& v6
        , const Dec_pnt& v7
        , const Pa_pnt& v8
        , const Lii_pnt& v9
        , const Bii_pnt& v10
        , const Plate_ra& v11
        , const Plate_dec& v12
        , const Plate_filter& v13
        , const Plate_id& v14
        , const Plate_origin& v15
        , const Observer& v16
        , const Object& v17
        , const Vvflags& v18
        , const Procrevision& v19
        , const Procdate& v20
        , const Sasversion& v21
        , const Ppsversion& v22
        , const Format& v23
        , const Name& v24
    ) : 
    _row(r)
    , _obs_mode(v0)
    , _obs_id(v1)
    , _obs_start(v2)
    , _obs_stop(v3)
    , _obs_duration(v4)
    , _content(v5)
    , _ra_pnt(v6)
    , _dec_pnt(v7)
    , _pa_pnt(v8)
    , _lii_pnt(v9)
    , _bii_pnt(v10)
    , _plate_ra(v11)
    , _plate_dec(v12)
    , _plate_filter(v13)
    , _plate_id(v14)
    , _plate_origin(v15)
    , _observer(v16)
    , _object(v17)
    , _vvflags(v18)
    , _procrevision(v19)
    , _procdate(v20)
    , _sasversion(v21)
    , _ppsversion(v22)
    , _format(v23)
    , _name(v24)
    { 
    }
    unsigned int row() const { return _row; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Plate_ra plate_ra() const { return _plate_ra; }
    Plate_dec plate_dec() const { return _plate_dec; }
    Plate_filter plate_filter() const { return _plate_filter; }
    Plate_id plate_id() const { return _plate_id; }
    Plate_origin plate_origin() const { return _plate_origin; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void plate_ra(const Plate_ra& v) { _plate_ra = v; }
    void plate_dec(const Plate_dec& v) { _plate_dec = v; }
    void plate_filter(const Plate_filter& v) { _plate_filter = v; }
    void plate_id(const Plate_id& v) { _plate_id = v; }
    void plate_origin(const Plate_origin& v) { _plate_origin = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Plate_ra _plate_ra;
    Plate_dec _plate_dec;
    Plate_filter _plate_filter;
    Plate_id _plate_id;
    Plate_origin _plate_origin;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Catsrc
: public std::vector<CatsrcRow>
{
    public:
    Catsrc() : std::vector<CatsrcRow>() {}
    Catsrc( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_EPICEXP
#define PINDEX_EPICEXP

class EpicexpRow
{
    public:
    EpicexpRow() {}
    EpicexpRow(
        const unsigned int& r
        , const Instrument& v0
        , const Exp_id& v1
        , const Mode& v2
        , const Datamode& v3
        , const Ep_filter& v4
        , const Obs_mode& v5
        , const Obs_id& v6
        , const Obs_start& v7
        , const Obs_stop& v8
        , const Obs_duration& v9
        , const Content& v10
        , const Ra_pnt& v11
        , const Dec_pnt& v12
        , const Pa_pnt& v13
        , const Lii_pnt& v14
        , const Bii_pnt& v15
        , const Exp_start& v16
        , const Exp_stop& v17
        , const Exp_duration& v18
        , const Band& v19
        , const Exposure& v20
        , const Srcdet& v21
        , const Ssp& v22
        , const Flarescreened& v23
        , const Observer& v24
        , const Object& v25
        , const Vvflags& v26
        , const Procrevision& v27
        , const Procdate& v28
        , const Sasversion& v29
        , const Ppsversion& v30
        , const Format& v31
        , const Name& v32
    ) : 
    _row(r)
    , _instrument(v0)
    , _exp_id(v1)
    , _mode(v2)
    , _datamode(v3)
    , _ep_filter(v4)
    , _obs_mode(v5)
    , _obs_id(v6)
    , _obs_start(v7)
    , _obs_stop(v8)
    , _obs_duration(v9)
    , _content(v10)
    , _ra_pnt(v11)
    , _dec_pnt(v12)
    , _pa_pnt(v13)
    , _lii_pnt(v14)
    , _bii_pnt(v15)
    , _exp_start(v16)
    , _exp_stop(v17)
    , _exp_duration(v18)
    , _band(v19)
    , _exposure(v20)
    , _srcdet(v21)
    , _ssp(v22)
    , _flarescreened(v23)
    , _observer(v24)
    , _object(v25)
    , _vvflags(v26)
    , _procrevision(v27)
    , _procdate(v28)
    , _sasversion(v29)
    , _ppsversion(v30)
    , _format(v31)
    , _name(v32)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Exp_id exp_id() const { return _exp_id; }
    Mode mode() const { return _mode; }
    Datamode datamode() const { return _datamode; }
    Ep_filter ep_filter() const { return _ep_filter; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Exp_start exp_start() const { return _exp_start; }
    Exp_stop exp_stop() const { return _exp_stop; }
    Exp_duration exp_duration() const { return _exp_duration; }
    Band band() const { return _band; }
    Exposure exposure() const { return _exposure; }
    Srcdet srcdet() const { return _srcdet; }
    Ssp ssp() const { return _ssp; }
    Flarescreened flarescreened() const { return _flarescreened; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void exp_id(const Exp_id& v) { _exp_id = v; }
    void mode(const Mode& v) { _mode = v; }
    void datamode(const Datamode& v) { _datamode = v; }
    void ep_filter(const Ep_filter& v) { _ep_filter = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void exp_start(const Exp_start& v) { _exp_start = v; }
    void exp_stop(const Exp_stop& v) { _exp_stop = v; }
    void exp_duration(const Exp_duration& v) { _exp_duration = v; }
    void band(const Band& v) { _band = v; }
    void exposure(const Exposure& v) { _exposure = v; }
    void srcdet(const Srcdet& v) { _srcdet = v; }
    void ssp(const Ssp& v) { _ssp = v; }
    void flarescreened(const Flarescreened& v) { _flarescreened = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Exp_id _exp_id;
    Mode _mode;
    Datamode _datamode;
    Ep_filter _ep_filter;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Exp_start _exp_start;
    Exp_stop _exp_stop;
    Exp_duration _exp_duration;
    Band _band;
    Exposure _exposure;
    Srcdet _srcdet;
    Ssp _ssp;
    Flarescreened _flarescreened;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Epicexp
: public std::vector<EpicexpRow>
{
    public:
    Epicexp() : std::vector<EpicexpRow>() {}
    Epicexp( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_EPICOBS
#define PINDEX_EPICOBS

class EpicobsRow
{
    public:
    EpicobsRow() {}
    EpicobsRow(
        const unsigned int& r
        , const Instrument& v0
        , const Obs_mode& v1
        , const Obs_id& v2
        , const Obs_start& v3
        , const Obs_stop& v4
        , const Obs_duration& v5
        , const Content& v6
        , const Ra_pnt& v7
        , const Dec_pnt& v8
        , const Pa_pnt& v9
        , const Lii_pnt& v10
        , const Bii_pnt& v11
        , const Observer& v12
        , const Object& v13
        , const Vvflags& v14
        , const Procrevision& v15
        , const Procdate& v16
        , const Sasversion& v17
        , const Ppsversion& v18
        , const Format& v19
        , const Name& v20
    ) : 
    _row(r)
    , _instrument(v0)
    , _obs_mode(v1)
    , _obs_id(v2)
    , _obs_start(v3)
    , _obs_stop(v4)
    , _obs_duration(v5)
    , _content(v6)
    , _ra_pnt(v7)
    , _dec_pnt(v8)
    , _pa_pnt(v9)
    , _lii_pnt(v10)
    , _bii_pnt(v11)
    , _observer(v12)
    , _object(v13)
    , _vvflags(v14)
    , _procrevision(v15)
    , _procdate(v16)
    , _sasversion(v17)
    , _ppsversion(v18)
    , _format(v19)
    , _name(v20)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Epicobs
: public std::vector<EpicobsRow>
{
    public:
    Epicobs() : std::vector<EpicobsRow>() {}
    Epicobs( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_EPICSRC
#define PINDEX_EPICSRC
class EpicsrcRow
{
    public:
    EpicsrcRow() {}
    EpicsrcRow(
        const unsigned int& r
        , const Instrument& v0
        , const Exp_id& v1
        , const Src_num& v2
        , const Band& v3
        , const Mode& v4
        , const Datamode& v5
        , const Ep_filter& v6
        , const Obs_mode& v7
        , const Obs_id& v8
        , const Obs_start& v9
        , const Obs_stop& v10
        , const Obs_duration& v11
        , const Content& v12
        , const Ra_pnt& v13
        , const Dec_pnt& v14
        , const Pa_pnt& v15
        , const Lii_pnt& v16
        , const Bii_pnt& v17
        , const Exp_start& v18
        , const Exp_stop& v19
        , const Exp_duration& v20
        , const Ra_obj& v21
        , const Dec_obj& v22
        , const Radec_err& v23
        , const Lii_obj& v24
        , const Bii_obj& v25
        , const Pn_cts& v26
        , const Pn_cts_err& v27
        , const M1_cts& v28
        , const M1_cts_err& v29
        , const M2_cts& v30
        , const M2_cts_err& v31
        , const Pn_tot_flux& v32
        , const Pn_tot_flux_err& v33
        , const M1_tot_flux& v34
        , const M1_tot_flux_err& v35
        , const M2_tot_flux& v36
        , const M2_tot_flux_err& v37
        , const Ep_tot_flux& v38
        , const Ep_tot_flux_err& v39
        , const Pn_tot& v40
        , const Pn_tot_err& v41
        , const M1_tot& v42
        , const M1_tot_err& v43
        , const M2_tot& v44
        , const M2_tot_err& v45
        , const Ep_tot& v46
        , const Ep_tot_err& v47
        , const Pn_det_ml& v48
        , const M1_det_ml& v49
        , const M2_det_ml& v50
        , const Ep_det_ml& v51
        , const Ep_extent& v52
        , const Ep_ext_err& v53
        , const Pn_ontime& v54
        , const M1_ontime& v55
        , const M2_ontime& v56
        , const Ep_hr1& v57
        , const Ep_hr1_err& v58
        , const Ep_hr2& v59
        , const Ep_hr2_err& v60
        , const Ep_hr3& v61
        , const Ep_hr3_err& v62
        , const Ep_hr4& v63
        , const Ep_hr4_err& v64
        , const Srcdet& v65
        , const Observer& v66
        , const Object& v67
        , const Ep_flag& v68
        , const M1_flag& v69
        , const M2_flag& v70
        , const Pn_flag& v71
        , const Vvflags& v72
        , const Procrevision& v73
        , const Procdate& v74
        , const Sasversion& v75
        , const Ppsversion& v76
        , const Format& v77
        , const Name& v78
    ) : 
    _row(r)
    , _instrument(v0)
    , _exp_id(v1)
    , _src_num(v2)
    , _band(v3)
    , _mode(v4)
    , _datamode(v5)
    , _ep_filter(v6)
    , _obs_mode(v7)
    , _obs_id(v8)
    , _obs_start(v9)
    , _obs_stop(v10)
    , _obs_duration(v11)
    , _content(v12)
    , _ra_pnt(v13)
    , _dec_pnt(v14)
    , _pa_pnt(v15)
    , _lii_pnt(v16)
    , _bii_pnt(v17)
    , _exp_start(v18)
    , _exp_stop(v19)
    , _exp_duration(v20)
    , _ra_obj(v21)
    , _dec_obj(v22)
    , _radec_err(v23)
    , _lii_obj(v24)
    , _bii_obj(v25)
    , _pn_cts(v26)
    , _pn_cts_err(v27)
    , _m1_cts(v28)
    , _m1_cts_err(v29)
    , _m2_cts(v30)
    , _m2_cts_err(v31)
    , _pn_tot_flux(v32)
    , _pn_tot_flux_err(v33)
    , _m1_tot_flux(v34)
    , _m1_tot_flux_err(v35)
    , _m2_tot_flux(v36)
    , _m2_tot_flux_err(v37)
    , _ep_tot_flux(v38)
    , _ep_tot_flux_err(v39)
    , _pn_tot(v40)
    , _pn_tot_err(v41)
    , _m1_tot(v42)
    , _m1_tot_err(v43)
    , _m2_tot(v44)
    , _m2_tot_err(v45)
    , _ep_tot(v46)
    , _ep_tot_err(v47)
    , _pn_det_ml(v48)
    , _m1_det_ml(v49)
    , _m2_det_ml(v50)
    , _ep_det_ml(v51)
    , _ep_extent(v52)
    , _ep_ext_err(v53)
    , _pn_ontime(v54)
    , _m1_ontime(v55)
    , _m2_ontime(v56)
    , _ep_hr1(v57)
    , _ep_hr1_err(v58)
    , _ep_hr2(v59)
    , _ep_hr2_err(v60)
    , _ep_hr3(v61)
    , _ep_hr3_err(v62)
    , _ep_hr4(v63)
    , _ep_hr4_err(v64)
    , _srcdet(v65)
    , _observer(v66)
    , _object(v67)
    , _ep_flag(v68)
    , _m1_flag(v69)
    , _m2_flag(v70)
    , _pn_flag(v71)
    , _vvflags(v72)
    , _procrevision(v73)
    , _procdate(v74)
    , _sasversion(v75)
    , _ppsversion(v76)
    , _format(v77)
    , _name(v78)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Exp_id exp_id() const { return _exp_id; }
    Src_num src_num() const { return _src_num; }
    Band band() const { return _band; }
    Mode mode() const { return _mode; }
    Datamode datamode() const { return _datamode; }
    Ep_filter ep_filter() const { return _ep_filter; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Exp_start exp_start() const { return _exp_start; }
    Exp_stop exp_stop() const { return _exp_stop; }
    Exp_duration exp_duration() const { return _exp_duration; }
    Ra_obj ra_obj() const { return _ra_obj; }
    Dec_obj dec_obj() const { return _dec_obj; }
    Radec_err radec_err() const { return _radec_err; }
    Lii_obj lii_obj() const { return _lii_obj; }
    Bii_obj bii_obj() const { return _bii_obj; }
    Pn_cts pn_cts() const { return _pn_cts; }
    Pn_cts_err pn_cts_err() const { return _pn_cts_err; }
    M1_cts m1_cts() const { return _m1_cts; }
    M1_cts_err m1_cts_err() const { return _m1_cts_err; }
    M2_cts m2_cts() const { return _m2_cts; }
    M2_cts_err m2_cts_err() const { return _m2_cts_err; }
    Pn_tot_flux pn_tot_flux() const { return _pn_tot_flux; }
    Pn_tot_flux_err pn_tot_flux_err() const { return _pn_tot_flux_err; }
    M1_tot_flux m1_tot_flux() const { return _m1_tot_flux; }
    M1_tot_flux_err m1_tot_flux_err() const { return _m1_tot_flux_err; }
    M2_tot_flux m2_tot_flux() const { return _m2_tot_flux; }
    M2_tot_flux_err m2_tot_flux_err() const { return _m2_tot_flux_err; }
    Ep_tot_flux ep_tot_flux() const { return _ep_tot_flux; }
    Ep_tot_flux_err ep_tot_flux_err() const { return _ep_tot_flux_err; }
    Pn_tot pn_tot() const { return _pn_tot; }
    Pn_tot_err pn_tot_err() const { return _pn_tot_err; }
    M1_tot m1_tot() const { return _m1_tot; }
    M1_tot_err m1_tot_err() const { return _m1_tot_err; }
    M2_tot m2_tot() const { return _m2_tot; }
    M2_tot_err m2_tot_err() const { return _m2_tot_err; }
    Ep_tot ep_tot() const { return _ep_tot; }
    Ep_tot_err ep_tot_err() const { return _ep_tot_err; }
    Pn_det_ml pn_det_ml() const { return _pn_det_ml; }
    M1_det_ml m1_det_ml() const { return _m1_det_ml; }
    M2_det_ml m2_det_ml() const { return _m2_det_ml; }
    Ep_det_ml ep_det_ml() const { return _ep_det_ml; }
    Ep_extent ep_extent() const { return _ep_extent; }
    Ep_ext_err ep_ext_err() const { return _ep_ext_err; }
    Pn_ontime pn_ontime() const { return _pn_ontime; }
    M1_ontime m1_ontime() const { return _m1_ontime; }
    M2_ontime m2_ontime() const { return _m2_ontime; }
    Ep_hr1 ep_hr1() const { return _ep_hr1; }
    Ep_hr1_err ep_hr1_err() const { return _ep_hr1_err; }
    Ep_hr2 ep_hr2() const { return _ep_hr2; }
    Ep_hr2_err ep_hr2_err() const { return _ep_hr2_err; }
    Ep_hr3 ep_hr3() const { return _ep_hr3; }
    Ep_hr3_err ep_hr3_err() const { return _ep_hr3_err; }
    Ep_hr4 ep_hr4() const { return _ep_hr4; }
    Ep_hr4_err ep_hr4_err() const { return _ep_hr4_err; }
    Srcdet srcdet() const { return _srcdet; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Ep_flag ep_flag() const { return _ep_flag; }
    M1_flag m1_flag() const { return _m1_flag; }
    M2_flag m2_flag() const { return _m2_flag; }
    Pn_flag pn_flag() const { return _pn_flag; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void exp_id(const Exp_id& v) { _exp_id = v; }
    void src_num(const Src_num& v) { _src_num = v; }
    void band(const Band& v) { _band = v; }
    void mode(const Mode& v) { _mode = v; }
    void datamode(const Datamode& v) { _datamode = v; }
    void ep_filter(const Ep_filter& v) { _ep_filter = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void exp_start(const Exp_start& v) { _exp_start = v; }
    void exp_stop(const Exp_stop& v) { _exp_stop = v; }
    void exp_duration(const Exp_duration& v) { _exp_duration = v; }
    void ra_obj(const Ra_obj& v) { _ra_obj = v; }
    void dec_obj(const Dec_obj& v) { _dec_obj = v; }
    void radec_err(const Radec_err& v) { _radec_err = v; }
    void lii_obj(const Lii_obj& v) { _lii_obj = v; }
    void bii_obj(const Bii_obj& v) { _bii_obj = v; }
    void pn_cts(const Pn_cts& v) { _pn_cts = v; }
    void pn_cts_err(const Pn_cts_err& v) { _pn_cts_err = v; }
    void m1_cts(const M1_cts& v) { _m1_cts = v; }
    void m1_cts_err(const M1_cts_err& v) { _m1_cts_err = v; }
    void m2_cts(const M2_cts& v) { _m2_cts = v; }
    void m2_cts_err(const M2_cts_err& v) { _m2_cts_err = v; }
    void pn_tot_flux(const Pn_tot_flux& v) { _pn_tot_flux = v; }
    void pn_tot_flux_err(const Pn_tot_flux_err& v) { _pn_tot_flux_err = v; }
    void m1_tot_flux(const M1_tot_flux& v) { _m1_tot_flux = v; }
    void m1_tot_flux_err(const M1_tot_flux_err& v) { _m1_tot_flux_err = v; }
    void m2_tot_flux(const M2_tot_flux& v) { _m2_tot_flux = v; }
    void m2_tot_flux_err(const M2_tot_flux_err& v) { _m2_tot_flux_err = v; }
    void ep_tot_flux(const Ep_tot_flux& v) { _ep_tot_flux = v; }
    void ep_tot_flux_err(const Ep_tot_flux_err& v) { _ep_tot_flux_err = v; }
    void pn_tot(const Pn_tot& v) { _pn_tot = v; }
    void pn_tot_err(const Pn_tot_err& v) { _pn_tot_err = v; }
    void m1_tot(const M1_tot& v) { _m1_tot = v; }
    void m1_tot_err(const M1_tot_err& v) { _m1_tot_err = v; }
    void m2_tot(const M2_tot& v) { _m2_tot = v; }
    void m2_tot_err(const M2_tot_err& v) { _m2_tot_err = v; }
    void ep_tot(const Ep_tot& v) { _ep_tot = v; }
    void ep_tot_err(const Ep_tot_err& v) { _ep_tot_err = v; }
    void pn_det_ml(const Pn_det_ml& v) { _pn_det_ml = v; }
    void m1_det_ml(const M1_det_ml& v) { _m1_det_ml = v; }
    void m2_det_ml(const M2_det_ml& v) { _m2_det_ml = v; }
    void ep_det_ml(const Ep_det_ml& v) { _ep_det_ml = v; }
    void ep_extent(const Ep_extent& v) { _ep_extent = v; }
    void ep_ext_err(const Ep_ext_err& v) { _ep_ext_err = v; }
    void pn_ontime(const Pn_ontime& v) { _pn_ontime = v; }
    void m1_ontime(const M1_ontime& v) { _m1_ontime = v; }
    void m2_ontime(const M2_ontime& v) { _m2_ontime = v; }
    void ep_hr1(const Ep_hr1& v) { _ep_hr1 = v; }
    void ep_hr1_err(const Ep_hr1_err& v) { _ep_hr1_err = v; }
    void ep_hr2(const Ep_hr2& v) { _ep_hr2 = v; }
    void ep_hr2_err(const Ep_hr2_err& v) { _ep_hr2_err = v; }
    void ep_hr3(const Ep_hr3& v) { _ep_hr3 = v; }
    void ep_hr3_err(const Ep_hr3_err& v) { _ep_hr3_err = v; }
    void ep_hr4(const Ep_hr4& v) { _ep_hr4 = v; }
    void ep_hr4_err(const Ep_hr4_err& v) { _ep_hr4_err = v; }
    void srcdet(const Srcdet& v) { _srcdet = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void ep_flag(const Ep_flag& v) { _ep_flag = v; }
    void m1_flag(const M1_flag& v) { _m1_flag = v; }
    void m2_flag(const M2_flag& v) { _m2_flag = v; }
    void pn_flag(const Pn_flag& v) { _pn_flag = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Exp_id _exp_id;
    Src_num _src_num;
    Band _band;
    Mode _mode;
    Datamode _datamode;
    Ep_filter _ep_filter;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Exp_start _exp_start;
    Exp_stop _exp_stop;
    Exp_duration _exp_duration;
    Ra_obj _ra_obj;
    Dec_obj _dec_obj;
    Radec_err _radec_err;
    Lii_obj _lii_obj;
    Bii_obj _bii_obj;
    Pn_cts _pn_cts;
    Pn_cts_err _pn_cts_err;
    M1_cts _m1_cts;
    M1_cts_err _m1_cts_err;
    M2_cts _m2_cts;
    M2_cts_err _m2_cts_err;
    Pn_tot_flux _pn_tot_flux;
    Pn_tot_flux_err _pn_tot_flux_err;
    M1_tot_flux _m1_tot_flux;
    M1_tot_flux_err _m1_tot_flux_err;
    M2_tot_flux _m2_tot_flux;
    M2_tot_flux_err _m2_tot_flux_err;
    Ep_tot_flux _ep_tot_flux;
    Ep_tot_flux_err _ep_tot_flux_err;
    Pn_tot _pn_tot;
    Pn_tot_err _pn_tot_err;
    M1_tot _m1_tot;
    M1_tot_err _m1_tot_err;
    M2_tot _m2_tot;
    M2_tot_err _m2_tot_err;
    Ep_tot _ep_tot;
    Ep_tot_err _ep_tot_err;
    Pn_det_ml _pn_det_ml;
    M1_det_ml _m1_det_ml;
    M2_det_ml _m2_det_ml;
    Ep_det_ml _ep_det_ml;
    Ep_extent _ep_extent;
    Ep_ext_err _ep_ext_err;
    Pn_ontime _pn_ontime;
    M1_ontime _m1_ontime;
    M2_ontime _m2_ontime;
    Ep_hr1 _ep_hr1;
    Ep_hr1_err _ep_hr1_err;
    Ep_hr2 _ep_hr2;
    Ep_hr2_err _ep_hr2_err;
    Ep_hr3 _ep_hr3;
    Ep_hr3_err _ep_hr3_err;
    Ep_hr4 _ep_hr4;
    Ep_hr4_err _ep_hr4_err;
    Srcdet _srcdet;
    Observer _observer;
    Object _object;
    Ep_flag _ep_flag;
    M1_flag _m1_flag;
    M2_flag _m2_flag;
    Pn_flag _pn_flag;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Epicsrc
: public std::vector<EpicsrcRow>
{
    public:
    Epicsrc() : std::vector<EpicsrcRow>() {}
    Epicsrc( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_INSTSUMM
#define PINDEX_INSTSUMM

class InstsummRow
{
    public:
    InstsummRow() {}
    InstsummRow(
        const unsigned int& r
        , const Instrument& v0
        , const Active& v1
        , const Nexposures& v2
        , const Priority& v3
    ) : 
    _row(r)
    , _instrument(v0)
    , _active(v1)
    , _nexposures(v2)
    , _priority(v3)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Active active() const { return _active; }
    Nexposures nexposures() const { return _nexposures; }
    Priority priority() const { return _priority; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void active(const Active& v) { _active = v; }
    void nexposures(const Nexposures& v) { _nexposures = v; }
    void priority(const Priority& v) { _priority = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Active _active;
    Nexposures _nexposures;
    Priority _priority;
};

class Instsumm
: public std::vector<InstsummRow>
{
    public:
    Instsumm() : std::vector<InstsummRow>() {}
    Instsumm( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_OMEXP
#define PINDEX_OMEXP
class OmexpRow
{
    public:
    OmexpRow() {}
    OmexpRow(
        const unsigned int& r
        , const Instrument& v0
        , const Detector& v1
        , const Om_filter& v2
        , const Mode& v3
        , const Datamode& v4
        , const Obs_mode& v5
        , const Obs_id& v6
        , const Obs_start& v7
        , const Obs_stop& v8
        , const Obs_duration& v9
        , const Exp_id& v10
        , const Content& v11
        , const Ra_pnt& v12
        , const Dec_pnt& v13
        , const Pa_pnt& v14
        , const Lii_pnt& v15
        , const Bii_pnt& v16
        , const Exp_start& v17
        , const Exp_stop& v18
        , const Exp_duration& v19
        , const Observer& v20
        , const Object& v21
        , const Vvflags& v22
        , const Procrevision& v23
        , const Procdate& v24
        , const Sasversion& v25
        , const Ppsversion& v26
        , const Format& v27
        , const Name& v28
    ) : 
    _row(r)
    , _instrument(v0)
    , _detector(v1)
    , _om_filter(v2)
    , _mode(v3)
    , _datamode(v4)
    , _obs_mode(v5)
    , _obs_id(v6)
    , _obs_start(v7)
    , _obs_stop(v8)
    , _obs_duration(v9)
    , _exp_id(v10)
    , _content(v11)
    , _ra_pnt(v12)
    , _dec_pnt(v13)
    , _pa_pnt(v14)
    , _lii_pnt(v15)
    , _bii_pnt(v16)
    , _exp_start(v17)
    , _exp_stop(v18)
    , _exp_duration(v19)
    , _observer(v20)
    , _object(v21)
    , _vvflags(v22)
    , _procrevision(v23)
    , _procdate(v24)
    , _sasversion(v25)
    , _ppsversion(v26)
    , _format(v27)
    , _name(v28)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Detector detector() const { return _detector; }
    Om_filter om_filter() const { return _om_filter; }
    Mode mode() const { return _mode; }
    Datamode datamode() const { return _datamode; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Exp_id exp_id() const { return _exp_id; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Exp_start exp_start() const { return _exp_start; }
    Exp_stop exp_stop() const { return _exp_stop; }
    Exp_duration exp_duration() const { return _exp_duration; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void detector(const Detector& v) { _detector = v; }
    void om_filter(const Om_filter& v) { _om_filter = v; }
    void mode(const Mode& v) { _mode = v; }
    void datamode(const Datamode& v) { _datamode = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void exp_id(const Exp_id& v) { _exp_id = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void exp_start(const Exp_start& v) { _exp_start = v; }
    void exp_stop(const Exp_stop& v) { _exp_stop = v; }
    void exp_duration(const Exp_duration& v) { _exp_duration = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Detector _detector;
    Om_filter _om_filter;
    Mode _mode;
    Datamode _datamode;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Exp_id _exp_id;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Exp_start _exp_start;
    Exp_stop _exp_stop;
    Exp_duration _exp_duration;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Omexp
: public std::vector<OmexpRow>
{
    public:
    Omexp() : std::vector<OmexpRow>() {}
    Omexp( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_OMOBS
#define PINDEX_OMOBS
class OmobsRow
{
    public:
    OmobsRow() {}
    OmobsRow(
        const unsigned int& r
        , const Instrument& v0
        , const Detector& v1
        , const Obs_mode& v2
        , const Obs_id& v3
        , const Obs_start& v4
        , const Obs_stop& v5
        , const Obs_duration& v6
        , const Content& v7
        , const Ra_pnt& v8
        , const Dec_pnt& v9
        , const Pa_pnt& v10
        , const Lii_pnt& v11
        , const Bii_pnt& v12
        , const Observer& v13
        , const Object& v14
        , const Vvflags& v15
        , const Procrevision& v16
        , const Procdate& v17
        , const Sasversion& v18
        , const Ppsversion& v19
        , const Format& v20
        , const Name& v21
    ) : 
    _row(r)
    , _instrument(v0)
    , _detector(v1)
    , _obs_mode(v2)
    , _obs_id(v3)
    , _obs_start(v4)
    , _obs_stop(v5)
    , _obs_duration(v6)
    , _content(v7)
    , _ra_pnt(v8)
    , _dec_pnt(v9)
    , _pa_pnt(v10)
    , _lii_pnt(v11)
    , _bii_pnt(v12)
    , _observer(v13)
    , _object(v14)
    , _vvflags(v15)
    , _procrevision(v16)
    , _procdate(v17)
    , _sasversion(v18)
    , _ppsversion(v19)
    , _format(v20)
    , _name(v21)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Detector detector() const { return _detector; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void detector(const Detector& v) { _detector = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Detector _detector;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};
class Omobs
: public std::vector<OmobsRow>
{
    public:
    Omobs() : std::vector<OmobsRow>() {}
    Omobs( Table* );
    std::string print() const;
};
#endif

#ifndef PINDEX_OMSRC
#define PINDEX_OMSRC

class OmsrcRow
{
    public:
    OmsrcRow() {}
    OmsrcRow(
        const unsigned int& r
        , const Instrument& v0
        , const Detector& v1
        , const Datamode& v2
        , const Mode& _Mode
        , const Om_filter& v3
        , const Obs_mode& v4
        , const Obs_id& v5
        , const Obs_start& v6
        , const Obs_stop& v7
        , const Obs_duration& v8
        , const Exp_id& v9
        , const Osw_id& v10
        , const Content& v11
        , const Ra_pnt& v12
        , const Dec_pnt& v13
        , const Pa_pnt& v14
        , const Lii_pnt& v15
        , const Bii_pnt& v16
        , const Exp_start& v17
        , const Exp_stop& v18
        , const Exp_duration& v19
        , const Src_num& v20
        , const Ra_obj& v21
        , const Dec_obj& v22
        , const Lii_obj& v23
        , const Bii_obj& v24
        , const Observer& v27
        , const Object& v28
        , const Vvflags& v29
        , const Procrevision& v30
        , const Procdate& v31
        , const Sasversion& v32
        , const Ppsversion& v33
        , const Format& v34
        , const Name& v35
    ) : 
    _row(r)
    , _instrument(v0)
    , _detector(v1)
    , _datamode(v2)
    , _mode(_Mode)
    , _om_filter(v3)
    , _obs_mode(v4)
    , _obs_id(v5)
    , _obs_start(v6)
    , _obs_stop(v7)
    , _obs_duration(v8)
    , _exp_id(v9)
    , _osw_id(v10)
    , _content(v11)
    , _ra_pnt(v12)
    , _dec_pnt(v13)
    , _pa_pnt(v14)
    , _lii_pnt(v15)
    , _bii_pnt(v16)
    , _exp_start(v17)
    , _exp_stop(v18)
    , _exp_duration(v19)
    , _src_num(v20)
    , _ra_obj(v21)
    , _dec_obj(v22)
    , _lii_obj(v23)
    , _bii_obj(v24)
    , _observer(v27)
    , _object(v28)
    , _vvflags(v29)
    , _procrevision(v30)
    , _procdate(v31)
    , _sasversion(v32)
    , _ppsversion(v33)
    , _format(v34)
    , _name(v35)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Detector detector() const { return _detector; }
    Datamode datamode() const { return _datamode; }
    Mode mode() const { return _mode; }
    Om_filter om_filter() const { return _om_filter; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Exp_id exp_id() const { return _exp_id; }
    Osw_id osw_id() const { return _osw_id; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Exp_start exp_start() const { return _exp_start; }
    Exp_stop exp_stop() const { return _exp_stop; }
    Exp_duration exp_duration() const { return _exp_duration; }
    Src_num src_num() const { return _src_num; }
    Ra_obj ra_obj() const { return _ra_obj; }
    Dec_obj dec_obj() const { return _dec_obj; }
    Lii_obj lii_obj() const { return _lii_obj; }
    Bii_obj bii_obj() const { return _bii_obj; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void detector(const Detector& v) { _detector = v; }
    void datamode(const Datamode& v) { _datamode = v; }
    void mode(const Mode& v) { _mode = v; }
    void om_filter(const Om_filter& v) { _om_filter = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void exp_id(const Exp_id& v) { _exp_id = v; }
    void osw_id(const Osw_id& v) { _osw_id = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void exp_start(const Exp_start& v) { _exp_start = v; }
    void exp_stop(const Exp_stop& v) { _exp_stop = v; }
    void exp_duration(const Exp_duration& v) { _exp_duration = v; }
    void src_num(const Src_num& v) { _src_num = v; }
    void ra_obj(const Ra_obj& v) { _ra_obj = v; }
    void dec_obj(const Dec_obj& v) { _dec_obj = v; }
    void lii_obj(const Lii_obj& v) { _lii_obj = v; }
    void bii_obj(const Bii_obj& v) { _bii_obj = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Detector _detector;
    Datamode _datamode;
    Mode _mode;
    Om_filter _om_filter;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Exp_id _exp_id;
    Osw_id _osw_id;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Exp_start _exp_start;
    Exp_stop _exp_stop;
    Exp_duration _exp_duration;
    Src_num _src_num;
    Ra_obj _ra_obj;
    Dec_obj _dec_obj;
    Lii_obj _lii_obj;
    Bii_obj _bii_obj;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};
class Omsrc
: public std::vector<OmsrcRow>
{
    public:
    Omsrc() : std::vector<OmsrcRow>() {}
    Omsrc( Table* );
    std::string print() const;
};
#endif
#ifndef PINDEX_OMSW
#define PINDEX_OMSW
class OmswRow
{
    public:
    OmswRow() {}
    OmswRow(
        const unsigned int& r
        , const Instrument& v0
        , const Detector& v1
        , const Om_filter& v2
        , const Mode& v3
        , const Datamode& v4
        , const Obs_mode& v5
        , const Obs_id& v6
        , const Obs_start& v7
        , const Obs_stop& v8
        , const Obs_duration& v9
        , const Exp_id& v10
        , const Osw_id& v11
        , const Content& v12
        , const Ra_pnt& v13
        , const Dec_pnt& v14
        , const Pa_pnt& v15
        , const Lii_pnt& v16
        , const Bii_pnt& v17
        , const Exp_start& v18
        , const Exp_stop& v19
        , const Exp_duration& v20
        , const Observer& v21
        , const Object& v22
        , const Vvflags& v23
        , const Procrevision& v24
        , const Procdate& v25
        , const Sasversion& v26
        , const Ppsversion& v27
        , const Format& v28
        , const Name& v29
    ) : 
    _row(r)
    , _instrument(v0)
    , _detector(v1)
    , _om_filter(v2)
    , _mode(v3)
    , _datamode(v4)
    , _obs_mode(v5)
    , _obs_id(v6)
    , _obs_start(v7)
    , _obs_stop(v8)
    , _obs_duration(v9)
    , _exp_id(v10)
    , _osw_id(v11)
    , _content(v12)
    , _ra_pnt(v13)
    , _dec_pnt(v14)
    , _pa_pnt(v15)
    , _lii_pnt(v16)
    , _bii_pnt(v17)
    , _exp_start(v18)
    , _exp_stop(v19)
    , _exp_duration(v20)
    , _observer(v21)
    , _object(v22)
    , _vvflags(v23)
    , _procrevision(v24)
    , _procdate(v25)
    , _sasversion(v26)
    , _ppsversion(v27)
    , _format(v28)
    , _name(v29)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Detector detector() const { return _detector; }
    Om_filter om_filter() const { return _om_filter; }
    Mode mode() const { return _mode; }
    Datamode datamode() const { return _datamode; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Exp_id exp_id() const { return _exp_id; }
    Osw_id osw_id() const { return _osw_id; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Exp_start exp_start() const { return _exp_start; }
    Exp_stop exp_stop() const { return _exp_stop; }
    Exp_duration exp_duration() const { return _exp_duration; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void detector(const Detector& v) { _detector = v; }
    void om_filter(const Om_filter& v) { _om_filter = v; }
    void mode(const Mode& v) { _mode = v; }
    void datamode(const Datamode& v) { _datamode = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void exp_id(const Exp_id& v) { _exp_id = v; }
    void osw_id(const Osw_id& v) { _osw_id = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void exp_start(const Exp_start& v) { _exp_start = v; }
    void exp_stop(const Exp_stop& v) { _exp_stop = v; }
    void exp_duration(const Exp_duration& v) { _exp_duration = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Detector _detector;
    Om_filter _om_filter;
    Mode _mode;
    Datamode _datamode;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Exp_id _exp_id;
    Osw_id _osw_id;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Exp_start _exp_start;
    Exp_stop _exp_stop;
    Exp_duration _exp_duration;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Omsw
: public std::vector<OmswRow>
{
    public:
    Omsw() : std::vector<OmswRow>() {}
    Omsw( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_PPSOBS
#define PINDEX_PPSOBS

class PpsobsRow
{
    public:
    PpsobsRow() {}
    PpsobsRow(
        const unsigned int& r
        , const Obs_mode& v0
        , const Obs_id& v1
        , const Obs_start& v2
        , const Obs_stop& v3
        , const Obs_duration& v4
        , const Content& v5
        , const Ra_pnt& v6
        , const Dec_pnt& v7
        , const Pa_pnt& v8
        , const Lii_pnt& v9
        , const Bii_pnt& v10
        , const Observer& v11
        , const Object& v12
        , const Vvflags& v13
        , const Procrevision& v14
        , const Procdate& v15
        , const Sasversion& v16
        , const Ppsversion& v17
        , const Format& v18
        , const Name& v19
    ) : 
    _row(r)
    , _obs_mode(v0)
    , _obs_id(v1)
    , _obs_start(v2)
    , _obs_stop(v3)
    , _obs_duration(v4)
    , _content(v5)
    , _ra_pnt(v6)
    , _dec_pnt(v7)
    , _pa_pnt(v8)
    , _lii_pnt(v9)
    , _bii_pnt(v10)
    , _observer(v11)
    , _object(v12)
    , _vvflags(v13)
    , _procrevision(v14)
    , _procdate(v15)
    , _sasversion(v16)
    , _ppsversion(v17)
    , _format(v18)
    , _name(v19)
    { 
    }
    unsigned int row() const { return _row; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Ppsobs
: public std::vector<PpsobsRow>
{
    public:
    Ppsobs() : std::vector<PpsobsRow>() {}
    Ppsobs( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_RGSEXP
#define PINDEX_RGSEXP
class RgsexpRow
{
    public:
    RgsexpRow() {}
    RgsexpRow(
        const unsigned int& r
        , const Instrument& v0
        , const Mode& v1
        , const Datamode& v2
        , const Obs_mode& v3
        , const Obs_id& v4
        , const Obs_start& v5
        , const Obs_stop& v6
        , const Obs_duration& v7
        , const Exp_id& v8
        , const Content& v9
        , const Ra_pnt& v10
        , const Dec_pnt& v11
        , const Pa_pnt& v12
        , const Lii_pnt& v13
        , const Bii_pnt& v14
        , const Exp_start& v15
        , const Exp_stop& v16
        , const Exp_duration& v17
	, const Filtgrps &_Filtgrps
        , const Ocb& v18
        , const Observer& v19
        , const Object& v20
        , const Vvflags& v21
        , const Procrevision& v22
        , const Procdate& v23
        , const Sasversion& v24
        , const Ppsversion& v25
        , const Format& v26
        , const Name& v27
    ) : 
    _row(r)
    , _instrument(v0)
    , _mode(v1)
    , _datamode(v2)
    , _obs_mode(v3)
    , _obs_id(v4)
    , _obs_start(v5)
    , _obs_stop(v6)
    , _obs_duration(v7)
    , _exp_id(v8)
    , _content(v9)
    , _ra_pnt(v10)
    , _dec_pnt(v11)
    , _pa_pnt(v12)
    , _lii_pnt(v13)
    , _bii_pnt(v14)
    , _exp_start(v15)
    , _exp_stop(v16)
    , _exp_duration(v17)
    , _filtgrps(_Filtgrps)
    , _ocb(v18)
    , _observer(v19)
    , _object(v20)
    , _vvflags(v21)
    , _procrevision(v22)
    , _procdate(v23)
    , _sasversion(v24)
    , _ppsversion(v25)
    , _format(v26)
    , _name(v27)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Mode mode() const { return _mode; }
    Datamode datamode() const { return _datamode; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Exp_id exp_id() const { return _exp_id; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Exp_start exp_start() const { return _exp_start; }
    Exp_stop exp_stop() const { return _exp_stop; }
    Exp_duration exp_duration() const { return _exp_duration; }
    Filtgrps filtgrps() const { return _filtgrps; }
    Ocb ocb() const { return _ocb; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void mode(const Mode& v) { _mode = v; }
    void datamode(const Datamode& v) { _datamode = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void exp_id(const Exp_id& v) { _exp_id = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void exp_start(const Exp_start& v) { _exp_start = v; }
    void exp_stop(const Exp_stop& v) { _exp_stop = v; }
    void exp_duration(const Exp_duration& v) { _exp_duration = v; }
    void filtgrps(const Filtgrps& v) { _filtgrps = v; }
    void ocb(const Ocb& v) { _ocb = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Mode _mode;
    Datamode _datamode;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Exp_id _exp_id;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Exp_start _exp_start;
    Exp_stop _exp_stop;
    Exp_duration _exp_duration;
    Filtgrps _filtgrps;
    Ocb _ocb;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Rgsexp
: public std::vector<RgsexpRow>
{
    public:
    Rgsexp() : std::vector<RgsexpRow>() {}
    Rgsexp( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_RGSOBS
#define PINDEX_RGSOBS

class RgsobsRow
{
    public:
    RgsobsRow() {}
    RgsobsRow(
        const unsigned int& r
        , const Instrument& v0
        , const Obs_mode& v1
        , const Obs_id& v2
        , const Obs_start& v3
        , const Obs_stop& v4
        , const Obs_duration& v5
        , const Content& v6
        , const Ra_pnt& v7
        , const Dec_pnt& v8
        , const Pa_pnt& v9
        , const Lii_pnt& v10
        , const Bii_pnt& v11
        , const Observer& v12
        , const Object& v13
        , const Vvflags& v14
        , const Procrevision& v15
        , const Procdate& v16
        , const Sasversion& v17
        , const Ppsversion& v18
        , const Format& v19
        , const Name& v20
    ) : 
    _row(r)
    , _instrument(v0)
    , _obs_mode(v1)
    , _obs_id(v2)
    , _obs_start(v3)
    , _obs_stop(v4)
    , _obs_duration(v5)
    , _content(v6)
    , _ra_pnt(v7)
    , _dec_pnt(v8)
    , _pa_pnt(v9)
    , _lii_pnt(v10)
    , _bii_pnt(v11)
    , _observer(v12)
    , _object(v13)
    , _vvflags(v14)
    , _procrevision(v15)
    , _procdate(v16)
    , _sasversion(v17)
    , _ppsversion(v18)
    , _format(v19)
    , _name(v20)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Rgsobs
: public std::vector<RgsobsRow>
{
    public:
    Rgsobs() : std::vector<RgsobsRow>() {}
    Rgsobs( Table* );
    std::string print() const;
};

#endif
#ifndef PINDEX_RGSSRC
#define PINDEX_RGSSRC
class RgssrcRow
{
    public:
    RgssrcRow() {}
    RgssrcRow(
        const unsigned int& r
        , const Instrument& v0
        , const Mode& v1
        , const Datamode& v2
        , const Obs_mode& v3
        , const Obs_id& v4
        , const Obs_start& v5
        , const Obs_stop& v6
        , const Obs_duration& v7
        , const Exp_id& v8
        , const Ocb& v9
        , const Content& v10
        , const Ra_pnt& v11
        , const Dec_pnt& v12
        , const Pa_pnt& v13
        , const Lii_pnt& v14
        , const Bii_pnt& v15
        , const Exp_start& v16
        , const Exp_stop& v17
        , const Exp_duration& v18
	, const Filtgrps& _Filtgrps
        , const Rgsorder& v19
        , const Src_num& v20
        , const Ra_obj& v21
        , const Dec_obj& v22
        , const Lii_obj& v23
        , const Bii_obj& v24
        , const Ep_tot& v25
        , const Ep_tot_err& v26
        , const Observer& v27
        , const Object& v28
        , const Vvflags& v29
        , const Procrevision& v30
        , const Procdate& v31
        , const Sasversion& v32
        , const Ppsversion& v33
        , const Format& v34
        , const Name& v35
    ) : 
    _row(r)
    , _instrument(v0)
    , _mode(v1)
    , _datamode(v2)
    , _obs_mode(v3)
    , _obs_id(v4)
    , _obs_start(v5)
    , _obs_stop(v6)
    , _obs_duration(v7)
    , _exp_id(v8)
    , _ocb(v9)
    , _content(v10)
    , _ra_pnt(v11)
    , _dec_pnt(v12)
    , _pa_pnt(v13)
    , _lii_pnt(v14)
    , _bii_pnt(v15)
    , _exp_start(v16)
    , _exp_stop(v17)
    , _exp_duration(v18)
    , _filtgrps(_Filtgrps)
    , _rgsorder(v19)
    , _src_num(v20)
    , _ra_obj(v21)
    , _dec_obj(v22)
    , _lii_obj(v23)
    , _bii_obj(v24)
    , _ep_tot(v25)
    , _ep_tot_err(v26)
    , _observer(v27)
    , _object(v28)
    , _vvflags(v29)
    , _procrevision(v30)
    , _procdate(v31)
    , _sasversion(v32)
    , _ppsversion(v33)
    , _format(v34)
    , _name(v35)
    { 
    }
    unsigned int row() const { return _row; }
    Instrument instrument() const { return _instrument; }
    Mode mode() const { return _mode; }
    Datamode datamode() const { return _datamode; }
    Obs_mode obs_mode() const { return _obs_mode; }
    Obs_id obs_id() const { return _obs_id; }
    Obs_start obs_start() const { return _obs_start; }
    Obs_stop obs_stop() const { return _obs_stop; }
    Obs_duration obs_duration() const { return _obs_duration; }
    Exp_id exp_id() const { return _exp_id; }
    Ocb ocb() const { return _ocb; }
    Content content() const { return _content; }
    Ra_pnt ra_pnt() const { return _ra_pnt; }
    Dec_pnt dec_pnt() const { return _dec_pnt; }
    Pa_pnt pa_pnt() const { return _pa_pnt; }
    Lii_pnt lii_pnt() const { return _lii_pnt; }
    Bii_pnt bii_pnt() const { return _bii_pnt; }
    Exp_start exp_start() const { return _exp_start; }
    Exp_stop exp_stop() const { return _exp_stop; }
    Exp_duration exp_duration() const { return _exp_duration; }
    Filtgrps filtgrps() const { return _filtgrps; }
    Rgsorder rgsorder() const { return _rgsorder; }
    Src_num src_num() const { return _src_num; }
    Ra_obj ra_obj() const { return _ra_obj; }
    Dec_obj dec_obj() const { return _dec_obj; }
    Lii_obj lii_obj() const { return _lii_obj; }
    Bii_obj bii_obj() const { return _bii_obj; }
    Ep_tot ep_tot() const { return _ep_tot; }
    Ep_tot_err ep_tot_err() const { return _ep_tot_err; }
    Observer observer() const { return _observer; }
    Object object() const { return _object; }
    Vvflags vvflags() const { return _vvflags; }
    Procrevision procrevision() const { return _procrevision; }
    Procdate procdate() const { return _procdate; }
    Sasversion sasversion() const { return _sasversion; }
    Ppsversion ppsversion() const { return _ppsversion; }
    Format format() const { return _format; }
    Name name() const { return _name; }
    void row(const unsigned int& r) { _row = r; }
    void instrument(const Instrument& v) { _instrument = v; }
    void mode(const Mode& v) { _mode = v; }
    void datamode(const Datamode& v) { _datamode = v; }
    void obs_mode(const Obs_mode& v) { _obs_mode = v; }
    void obs_id(const Obs_id& v) { _obs_id = v; }
    void obs_start(const Obs_start& v) { _obs_start = v; }
    void obs_stop(const Obs_stop& v) { _obs_stop = v; }
    void obs_duration(const Obs_duration& v) { _obs_duration = v; }
    void exp_id(const Exp_id& v) { _exp_id = v; }
    void ocb(const Ocb& v) { _ocb = v; }
    void content(const Content& v) { _content = v; }
    void ra_pnt(const Ra_pnt& v) { _ra_pnt = v; }
    void dec_pnt(const Dec_pnt& v) { _dec_pnt = v; }
    void pa_pnt(const Pa_pnt& v) { _pa_pnt = v; }
    void lii_pnt(const Lii_pnt& v) { _lii_pnt = v; }
    void bii_pnt(const Bii_pnt& v) { _bii_pnt = v; }
    void exp_start(const Exp_start& v) { _exp_start = v; }
    void exp_stop(const Exp_stop& v) { _exp_stop = v; }
    void exp_duration(const Exp_duration& v) { _exp_duration = v; }
    void filtgrps(const Filtgrps& v) { _filtgrps = v; }
    void rgsorder(const Rgsorder& v) { _rgsorder = v; }
    void src_num(const Src_num& v) { _src_num = v; }
    void ra_obj(const Ra_obj& v) { _ra_obj = v; }
    void dec_obj(const Dec_obj& v) { _dec_obj = v; }
    void lii_obj(const Lii_obj& v) { _lii_obj = v; }
    void bii_obj(const Bii_obj& v) { _bii_obj = v; }
    void ep_tot(const Ep_tot& v) { _ep_tot = v; }
    void ep_tot_err(const Ep_tot_err& v) { _ep_tot_err = v; }
    void observer(const Observer& v) { _observer = v; }
    void object(const Object& v) { _object = v; }
    void vvflags(const Vvflags& v) { _vvflags = v; }
    void procrevision(const Procrevision& v) { _procrevision = v; }
    void procdate(const Procdate& v) { _procdate = v; }
    void sasversion(const Sasversion& v) { _sasversion = v; }
    void ppsversion(const Ppsversion& v) { _ppsversion = v; }
    void format(const Format& v) { _format = v; }
    void name(const Name& v) { _name = v; }
    std::string print() const;
    private:
    int _row;
    Instrument _instrument;
    Mode _mode;
    Datamode _datamode;
    Obs_mode _obs_mode;
    Obs_id _obs_id;
    Obs_start _obs_start;
    Obs_stop _obs_stop;
    Obs_duration _obs_duration;
    Exp_id _exp_id;
    Ocb _ocb;
    Content _content;
    Ra_pnt _ra_pnt;
    Dec_pnt _dec_pnt;
    Pa_pnt _pa_pnt;
    Lii_pnt _lii_pnt;
    Bii_pnt _bii_pnt;
    Exp_start _exp_start;
    Exp_stop _exp_stop;
    Exp_duration _exp_duration;
    Filtgrps _filtgrps;
    Rgsorder _rgsorder;
    Src_num _src_num;
    Ra_obj _ra_obj;
    Dec_obj _dec_obj;
    Lii_obj _lii_obj;
    Bii_obj _bii_obj;
    Ep_tot _ep_tot;
    Ep_tot_err _ep_tot_err;
    Observer _observer;
    Object _object;
    Vvflags _vvflags;
    Procrevision _procrevision;
    Procdate _procdate;
    Sasversion _sasversion;
    Ppsversion _ppsversion;
    Format _format;
    Name _name;
};

class Rgssrc
: public std::vector<RgssrcRow>
{
    public:
    Rgssrc() : std::vector<RgssrcRow>() {}
    Rgssrc( Table* );
    std::string print() const;
};
#endif
#ifndef PINDEX
#define PINDEX

class Pindex
{
    public:
    Pindex( const std::string& );
    std::string print() const;
    const Catcat* catcat() const { return &_catcat; }
    void catcat (const Catcat& v) { _catcat = v; }
    const Catobs* catobs() const { return &_catobs; }
    void catobs (const Catobs& v) { _catobs = v; }
    const Catsrc* catsrc() const { return &_catsrc; }
    void catsrc (const Catsrc& v) { _catsrc = v; }
    const Epicexp* epicexp() const { return &_epicexp; }
    void epicexp (const Epicexp& v) { _epicexp = v; }
    const Epicobs* epicobs() const { return &_epicobs; }
    void epicobs (const Epicobs& v) { _epicobs = v; }
    const Epicsrc* epicsrc() const { return &_epicsrc; }
    void epicsrc (const Epicsrc& v) { _epicsrc = v; }
    const Instsumm* instsumm() const { return &_instsumm; }
    void instsumm (const Instsumm& v) { _instsumm = v; }
    const Omexp* omexp() const { return &_omexp; }
    void omexp (const Omexp& v) { _omexp = v; }
    const Omobs* omobs() const { return &_omobs; }
    void omobs (const Omobs& v) { _omobs = v; }
    const Omsrc* omsrc() const { return &_omsrc; }
    void omsrc (const Omsrc& v) { _omsrc = v; }
    const Omsw* omsw() const { return &_omsw; }
    void omsw (const Omsw& v) { _omsw = v; }
    const Ppsobs* ppsobs() const { return &_ppsobs; }
    void ppsobs (const Ppsobs& v) { _ppsobs = v; }
    const Rgsexp* rgsexp() const { return &_rgsexp; }
    void rgsexp (const Rgsexp& v) { _rgsexp = v; }
    const Rgsobs* rgsobs() const { return &_rgsobs; }
    void rgsobs (const Rgsobs& v) { _rgsobs = v; }
    const Rgssrc* rgssrc() const { return &_rgssrc; }
    void rgssrc (const Rgssrc& v) { _rgssrc = v; }

    // Used to avoid unused variable errors when compiling.
    int dummy() const { return 1; }

    private:
    std::string _filename;
    Catcat _catcat;
    Catobs _catobs;
    Catsrc _catsrc;
    Epicexp _epicexp;
    Epicobs _epicobs;
    Epicsrc _epicsrc;
    Instsumm _instsumm;
    Omexp _omexp;
    Omobs _omobs;
    Omsrc _omsrc;
    Omsw _omsw;
    Ppsobs _ppsobs;
    Rgsexp _rgsexp;
    Rgsobs _rgsobs;
    Rgssrc _rgssrc;
};

#endif

#endif
