// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMM_PPSINDEX
#define PPSSUMM_PPSINDEX

#include <iostream>
#include <string>
#include <vector>
#include <map>
#include <algorithm>
#include <iterator>
#include <sstream>

// SAS 1
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"

// PPSSUMM Type classes
#include "ppssumm_typedef.h"
#include "string_algo.h"
#include "key_algo.h"
#include "PpssummCommon.h"
#include "Filename_typedef.h"
#include "Filename.h"
#include "Instrument.h"
#include "Src_num.h"
#include "Band.h"
#include "Rgsorder.h"
#include "Degrees.h"
#include "Datamode.h"
#include "Pindex_typedef.h"
#include "Pindex.h"


class GenericproductRow
{
    public:
    GenericproductRow( const unsigned int v0 ,  const Name& v1 , const Content& v2 ) 
    : _row(v0) , _name(v1) , _content(v2) 
    {
    }

    bool operator<(const GenericproductRow& e) const
    {
	return _name < e.name();
    }

    friend ostream& operator<<( ostream& os , const GenericproductRow& r )
    {
	os  << "<tr>"
	    << "<td class=\"string\">" 
	    << r._name 
	    << "</td>"
	    << "<td class=\"string\">" 
	    << r._content 
	    << "</td>"
	    << "</tr>"
	    << "\n"
	;
	return os;
    }
    Name name() const { return _name; }
    Content content() const { return _content; }

    private:
    unsigned int _row;
    Name _name;
    Content _content;
};

class Genericproduct : public std::vector<GenericproductRow>
{
    public:
    Genericproduct() : std::vector<GenericproductRow>()  {}
    Genericproduct(const Pindex& pindex) : std::vector<GenericproductRow>() 
    {
	const Ppsobs* ppsobs = pindex.ppsobs();
	unsigned long row=0;
	for ( Ppsobs::const_iterator CI = ppsobs->begin()
	    ; CI != ppsobs->end()
	    ; ++CI
	){
	    this->push_back( GenericproductRow( row++ , CI->name() , CI->content() ) );
	}

	std::sort(this->begin(), this->end() , std::less<GenericproductRow>() );
    }

    void title( ostream& os ) const
    {
	os  << "<tr><th class=\"string\">Filename</th><th class=\"string\">Content</th></tr>"
	    << "\n"
	;
    }
    friend ostream& operator<<( ostream& os , const Genericproduct& r )
    {
	r.title(os);
	for( Genericproduct::const_iterator CI = r.begin()
	    ; CI != r.end()
	    ; ++CI
	){
	    os << *CI;
	}
	return os;
    }
    private:
};

#endif
