// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "PpssummCommon.h"

void PpssummCommon::extract_data( DataSet* fitsum , const std::string& pindex_file )
{
    //Pindex class does not contain attributes yet.

    DataSet* pidx = dataSetServer->open( pindex_file , Dal::Read);
    Table* tbl = pidx->table(0);
    if (tbl->hasAttribute("PROCDATE") ) {
        procdate( tbl->attribute("PROCDATE")->asString() );
    } else {
         procdate( "Not Available" );
    }
    dataSetServer->close(pidx);

    stringlist collect;
    std::string tname("OBSERVAT");
    if ( fitsum->hasBlock(tname) ) 
    {
        Table* tbl = fitsum->table(tname);
	stringlist collect;
        
        const std::string want[] = 
	{
             "OBS_ID"
            ,"REVOLUTION"
            ,"OBS_START"
            ,"OBS_STOP"
            ,""
        };
        const std::string* w = want;

        while ( w->length() ) 
	{
            if ( tbl->hasColumn(*w) ) 
	    {
                stringColumn(tbl , *w , back_inserter( collect ) );
            }
	    else
	    {
                errstr << "Unable to find column "
                    << *w << " in table OBSERVAT of summary file."
                    << Sas::error("ppssumm:BADSUMMARYFILE")
		;        
            }
            w++;
        }
	obs_id(collect[0]);
	rev(collect[1]);
	obs_start(collect[2]);
	obs_stop(collect[3]);

	double d = STime(collect[3]) - STime(collect[2]);
	std::stringstream ss;
	ss << d;
	sched( ss.str() );
    } else {
        errstr << "Unable to find OBSERVAT table in sumary file" << Sas::error("BADSUMMARYFILE");
    }

    collect.clear();
    tname = "PROPOSAL";
    if ( fitsum->hasBlock(tname) ) {

        Table* tbl = fitsum->table(tname);   
        const std::string want[] = {
             "TARGET"
            ,"GONAME"
            ,""
        };
        const std::string* w = want;

        while ( w->length() ) 
	{
            if ( tbl->hasColumn(*w) ) 
	    {
                stringColumn(tbl , *w , back_inserter( collect ) );
            }
	    else
	    {
                errstr << "Unable to find column "
                    << *w << " in table PROPOSAL of summary file."
                    << Sas::error("ppssumm:BADSUMMARYFILE")
		;        
            }
            w++;
        }
	target(collect[0]);
	goname(collect[1]);
    }
    else
    {
        errstr << "Unable to find PROPOSAL table in sumary file" << Sas::error("BADSUMMARYFILE");
    }

    collect.clear();
    tname = "CONFIG";
    if ( fitsum->hasBlock(tname) ) {

        Table* tbl = fitsum->table(tname);   
        const std::string want[] = {
             "ODF_VERSION"
            ,""
        };
        const std::string* w = want;

        while ( w->length() ) 
	{
            if ( tbl->hasColumn(*w) ) 
	    {
                stringColumn(tbl , *w , back_inserter( collect ) );
            }
	    else
	    {
                errstr << "Unable to find column "
                    << *w << " in table CONFIG of summary file."
                    << Sas::error("ppssumm:BADSUMMARYFILE")
		;        
            }
            w++;
        }
	odf_version(collect[0]);
    }
    else
    {
	odf_version("001");
        errstr << "Unable to find CONFIG table in sumary file" << Sas::warning("BADSUMMARYFILE");
    }


}

