// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMMS_SIMPLEATTRIBUTE
#define PPSSUMMS_SIMPLEATTRIBUTE

#include <string>
#include <ostream>
#include "stripws.h"

#include "Dal.h"

class SAttribute 
{
    // Simple class to store what can be turned into an Attribute
    // Also usefull for storing split summary lines.
public:
    SAttribute( const std::string& n="" 
        , const std::string& v="" 
        , const std::string& c=""
    ) : name_(n) , value_(v) , comment_(c) {}
    
    std::string name() const { return name_;}
    SAttribute name(const std::string& n) 
    { name_ = n; return *this; }

    std::string value() const { return value_; }
    SAttribute& value(const std::string& v) 
    { value_ = v; return *this;}

    std::string comment() const { return comment_; }
    SAttribute  comment(const std::string& c)
    { comment_ = c ; return *this; }

    // strictly speeking an attribute name is at most 8 characters long
    std::string name8() const 
    { return (name_.length() > 8) ? name_.substr(0,8) : name_ ; }

    friend std::ostream &operator<<(std::ostream& os ,  const SAttribute& att)
    {
        os  << "Name/Value/Comment = " 
            << att.name_ << "/" << att.value_ << "/" << att.comment_ << std::endl;
        return os;
    }
private:
    std::string name_;
    std::string value_;
    std::string comment_;


};

#endif
