// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMMS_SIMPLECOLUMN
#define PPSSUMMS_SIMPLECOLUMN

#include <string>
#include <vector>
#include <ostream>
#include "SAttribute.h"
#include "stripws.h"

#include "Dal.h"

class SColumn : public std::vector<std::string>
{
    // A container for table columns
public:
    SColumn(const std::string& n="" , const std::string& c="") 
    : name_(n) , comment_(c)
    {
        clear();
    }

    explicit SColumn(const SAttribute& sa) 
    : std::vector<std::string>(1,sa.value()) , name_(sa.name()) , comment_(sa.comment())
    {}

    std::string name() const { return name_; }
    SColumn& name(const std::string& n)
    { name_ = n ; return *this ; }
    
    std::string comment() const { return comment_; }
    SColumn& comment(const std::string& c)
    { comment_ = c ; return *this ; }

    std::string::size_type width() const
    {
        /* Return length of longest string held in the container */
        std::string::size_type width = 0;
        for(const_iterator CI = begin() ; CI != end() ; CI++) {
            if ( CI->size() > width) width =  CI->size(); 
        }
        return width;
    }

    void pad(unsigned int n , std::string::size_type wdth=0 )
    {
        /* Add n blank rows to a column of width wdth */
        std::string::size_type w = (wdth > width()) ? wdth : width();

        while (n-- && w) std::vector<std::string>::push_back( std::string(w , '\0' ) );
        
    }
    
    void push_back(const SAttribute& sa) 
    {
        /* Add the value_ of an SAttribute to the column */

        std::vector<std::string>::push_back(sa.value()); 
    }

    void insert(SColumn::const_iterator begin_ , SColumn::const_iterator end_) 
    {
        while (begin_ != end_) {
            std::vector<std::string>::push_back( *begin_++ );
        }
        
    }

    friend std::ostream &operator<<(std::ostream& os , const SColumn& sc)
    {
        os << "Name/Comment/Length/Width = "
        << sc.name_ << "/" << sc.comment_ << "/" << sc.size() << "/" << sc.width() << std::endl;
        return os;

    }
private:
    std::string name_;
    std::string comment_;

};

#endif
