// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMM_SRC_NUM
#define PPSSUMM_SRC_NUM

#include <string>
#include <sstream>
#include <cstdlib>
#include <iostream>
#include "BaseInteger.h"
#include "Dal.h"

class Src_num : public BaseInteger<int16>
{
    public:
    Src_num() : BaseInteger<int16>(0) { }
    Src_num( const int16& s ) : BaseInteger<int16>(s) {  }
    Src_num( const char* s ) : BaseInteger<int16>( static_cast<int16>( strtol( s , nullptr , 16 ) ) ) {  }
    Src_num( const std::string& s ) :  BaseInteger<int16>( static_cast<int16>( strtol( s.c_str() , nullptr , 16 ) ) ) {  }

    // Switch between hex / decimal output
    void hexon() { _hex = 1; }
    void hexoff() { _hex = 0; }

    Src_num operator=(const char* s)
    {
        this->set( static_cast<int16>( strtol( s , nullptr , 16 ) ) );
        return *this;
    }
    Src_num operator=(const std::string& s)
    { 
        this->set( static_cast<int16>( strtol( s.c_str() , nullptr , 16 ) ) );
        return *this;
    }

    ostream& asHex( ostream& os ) const 
    {
	std::stringstream ss;
	ss.setf( std::ios::hex | std::ios::uppercase , ios::basefield );
	ss.setf( std::ios::uppercase );
	ss.fill('0');
	ss.width(3);
        int16 tmp = this->get();
	ss << tmp;
	os << ss.str();
	return os;
    }
    friend std::ostream &operator<<(std::ostream& os , const Src_num& s)
    {
        /*
         * If _hex is true tweak the stream before output then return it to it's previous state afterwards.
         * The previous state may have been ios::hex so use the old state returned by setf just in case.
         * If hex is false do not tweak the stream  -assume the user has set it up how they want it.
         */
        int16 tmp = s.get();
        if (!tmp)
        {
	    os << "&nbsp;";
        }
        else
        {
	    if (_hex)
	    {
		s.asHex(os);
	    }
	    else
	    {
		os << tmp;
	    }
        }
	return os;
    }

    private:
    static bool _hex;
};

//bool Src_num::_hex = 0;

class MakeSrc_num
{
    public:
    Src_num operator()( const Src_num& s ) { return Src_num( s.get() ); }
    Src_num operator()( const int16&  s ) { return Src_num(s); }
    Src_num operator()( const char* s ) { return Src_num(s); }
    Src_num operator()( const std::string& s ) { return Src_num(s); }
};

#endif
