// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef PPSSUMM_GETSUMFILE
#define PPSSUMM_GETSUMFILE

#include <string>
#include <cstdlib>
#include <dirent.h>
#include "Tracer.h"
#include "errstr.h"


std::string getSumFile(std::string dirName="")
{
    /*
     * Based on getSumFile from oal/.../Odf.cc where they declare it static
     */

    if (! dirName.length() ) {
        
        char* p = getenv("SAS_ODF");
        if ( p == NULL ) {
            errstr << "SAS_ODF not defined."
                  << Sas::fatal("Sum2Fit:SAS_ODFUNDEFINED")
            ;
        } else {
            dirName = p;
        }
    }
    
    errstr << "dirName " << dirName << Sas::message(Msg::App, Msg::Sparse);


        DIR *dirp = opendir(dirName.c_str());
        if (!dirp)
                errstr << "Cannot access directory given by SAS_ODF environment variable."
                          << Sas::fatal("Sum2Fit:DIRNOTFOUND")
        ;

    std::string sumSetName;
    
        while (struct dirent *dp = readdir(dirp)) {
                string s(dp->d_name);
                if (s.length() >= 8)
                        s.erase(0, s.length()-7);

                if (s == "SUM.SAS") {
                        sumSetName = dp->d_name;
                        continue;
                } else if (s == "SUM.ASC") {
                        sumSetName = dp->d_name;
                        continue;
                }
        }
        closedir(dirp);

        if (sumSetName.empty()) {
        errstr << "Cannot find either a SUM.SAS or SUM.ASC format summary file"
            << "In the directory given by SAS_ODF"
            << Sas::fatal("Sum2Fit:SUMMARYFILENOTFOUND")
        ;
        
    }
 
    errstr << "sumSetName " << sumSetName << Sas::message(Msg::App, Msg::Sparse);
        return (dirName + "/") + sumSetName;
}

#endif
