// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "key_algo.h"

/*
 * These functions are used to make keys 
 * Those without filenames tend to get used in map<> and <a name="">
 * Those with filenames tend to be used in <a href="">
 */

namespace ppssumm_internal
{


std::string makekey( const Instrument& i , const Exp_id& e , const Src_num& s )
{
    	std::stringstream ss;
	ss << "i" << i << "e" << e << "s" << s ;
	return ss.str();
}

std::string makekey( const Instrument& i , const Exp_id& e , const Osw_id& q , const Src_num& s  )
{
    	std::stringstream ss;
	ss << "i" << i << "e" << e << "q" << q << "s" << s ;
	return ss.str();
}

std::string makekey( const Instrument& i , const Exp_id& e , const Osw_id& q )
{
    	std::stringstream ss;
	ss << "i" << i << "e" << e << "q" << q ;
	return ss.str();
}

std::string makekey( const Instrument& i , const Exp_id& e )
{
    	std::stringstream ss;
	ss << "i" << i << "e" << e ;
	return ss.str();
}

std::string makekey( const Filename& f 
    , const Instrument& i 
    , const Exp_id& e 
    , const Src_num& s 
){
    	std::stringstream ss;
	ss << f.asString() << "#" << makekey( i , e , s ) ;
	return ss.str();
}

std::string makekey( const Filename& f 
    , const Instrument& i 
    , const Exp_id& e 
    , const Osw_id& q 
    , const Src_num& s
){
    	std::stringstream ss;
	ss << f.asString() << "#" << makekey( i , e , q , s ) ;
	return ss.str();
}

std::string makekey( const Filename& f 
    , const Instrument& i 
    , const Exp_id& e 
    , const Osw_id& q 
){
    	std::stringstream ss;
	ss << f.asString() << "#" << makekey( i , e , q ) ;
	return ss.str();
}

std::string makekey( const Filename& f 
    , const Instrument& i 
    , const Exp_id& e 
){
    	std::stringstream ss;
	ss << f.asString() << "#" << makekey( i , e ) ;
	return ss.str();
}

}
