// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "ppssumm_fits.h"

stringlist severalsingleton( const DataSet* ds , const std::string& t , const std::string* w )
{
    // Extract several column values from a table expected to have 1 row
    // The last element in std::string[] must be empty.
    std::string deflt("");
    stringlist r;
    stringlist collect;
    if (ds->hasBlock(t))
    {
	const Table* tbl = ds->table(t);
	while (w->length())
	{
	    collect.clear();
	    if ( tbl->hasColumn(*w) )
	    {
		stringColumn(tbl , *w , back_inserter( collect ) );
		if (collect.size())
			r.push_back(collect[0]);
		else
			r.push_back(deflt);
	    }
	    w++;
	}
    }
    return r;
}

stringlist severalsingleton( const DataSet* ds , const std::string& t , stringlist_ci first , stringlist_ci last)
{
    // Extract several column values from a table expected to have 1 row
    
    std::string deflt("");
    stringlist r;
    stringlist collect;
    if (ds->hasBlock(t))
    {
	const Table* tbl = ds->table(t);
	while (first != last)
	{
	    collect.clear();
	    if ( tbl->hasColumn(*first) )
	    {
		stringColumn(tbl , *first , back_inserter( collect ) );
		if (collect.size())
			r.push_back(collect[0]);
		else
			r.push_back(deflt);
	    }
	    ++first;
	}
    }
    return r;
}

std::string singleton( const DataSet* ds , const std::string& t , const std::string& c)
{
    // Extract a column value from a table expected to have 1 row
    std::string r("");
    stringlist collect;
    if (ds->hasBlock(t))
    {
	const Table* tbl = ds->table(t);
	if ( tbl->hasColumn(c) )
	{
	    stringColumn(tbl , c , back_inserter( collect ) );
	    r = collect[0];
	}
    }
    return r;
}

